from enum import Enum


class RuleMode(str, Enum):
    # Rule modes
    BLOCKING = "BLOCKING"
    MONITOR = "MONITOR"
    DEBUG = "DEBUG"
    ALLOWED = "ALLOWED"
    DISABLED = "DISABLED"


class RuleRisk(str, Enum):
    # Rule risk severity
    LOW = "LOW"
    MEDIUM = "MEDIUM"
    HIGH = "HIGH"


class RuleGranularity(str, Enum):
    # Rule
    ACTION = "ACTION"
    RESOURCE = "RESOURCE"
    STACK = "STACK"



__all__ = ["CloudFormationAuthenticationRule"]

from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel

from cfripper.model.enums import RuleGranularity
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule


class CloudFormationAuthenticationRule(Rule):
    """
    Checks that any `AWS::CloudFormation::Authentication` resource does not contain plain text credentials.

    Risk:
        Secrets are stored in clear text and printed in clear text in the AWS console.

    Fix:
        Do not store credentials in CloudFormation files, use parameters.

    Code for fix:
        ````yml
        Parameters:
          PasswordAuth:
            NoEcho: true
            Description: Some cool password
            MinLength: 8
            Type: String

        ...

        Resources:
          AWS::CloudFormation::Authentication:
            ...
            password:
              Ref: "PasswordAuth"
            ...
        ````

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `Resource`                       | Resource that is being addressed                               |
    """

    REASON = "Hardcoded credentials in {}"
    GRANULARITY = RuleGranularity.RESOURCE

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if resource.has_hardcoded_credentials():
                self.add_failure_to_result(
                    result,
                    self.REASON.format(logical_id),
                    resource_ids={logical_id},
                    context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
                )
        return result



__all__ = ["S3LifecycleConfigurationRule"]

from typing import Dict, Optional

from pycfmodel.model.resources.s3_bucket import S3Bucket

from cfripper.model.enums import RuleGranularity, RuleRisk
from cfripper.model.result import Result
from cfripper.rules.base_rules import ResourceSpecificRule


class S3LifecycleConfigurationRule(ResourceSpecificRule):
    """
    Checks for the presence of `LifecycleConfiguration` on S3 buckets.
    These rules can help with security, compliance, and reduce AWS Costs. The rule does not
    check the specific rules contained with the `LifecycleConfiguration` key.

    Fix:
        Add `LifecycleConfiguration` property to the S3 Bucket as defined in
        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig.html.

    Code for fix:
        An example rule is included within the configuration.

        ````yml
        Resources:
          S3Bucket:
            Type: AWS::S3::Bucket
            Properties:
              ...
              LifecycleConfiguration:
                Rules:
                  - Status: Enabled
                    Prefix: logs/
                    ExpirationInDays: 7
              ...
        ````

    Filters context:
        | Parameter     | Type               | Description                                                    |
        |:-------------:|:------------------:|:--------------------------------------------------------------:|
        |`config`       | str                | `config` variable available inside the rule                    |
        |`extras`       | str                | `extras` variable available inside the rule                    |
        |`logical_id`   | str                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`     | `S3Bucket`         | Resource that is being addressed                               |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "S3 Bucket {} is required to contain a LifecycleConfiguration property"
    RESOURCE_TYPES = (S3Bucket,)
    RISK_VALUE = RuleRisk.LOW

    def resource_invoke(self, resource: S3Bucket, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()
        if not resource.Properties.LifecycleConfiguration:
            self.add_failure_to_result(
                result,
                self.REASON.format(logical_id),
                resource_ids={logical_id},
                context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
            )
        return result



import importlib
import itertools
import logging
import sys
from collections import defaultdict
from io import TextIOWrapper
from pathlib import Path
from typing import DefaultDict, Dict, List

from pydantic import BaseModel

from cfripper.config.constants import (
    AWS_CLOUDTRAIL_ACCOUNT_IDS,
    AWS_ELASTICACHE_BACKUP_CANONICAL_IDS,
    AWS_ELB_LOGS_ACCOUNT_IDS,
    AWS_REDSHIFT_AUDIT_ACCOUNT_IDS,
)

from .filter import Filter
from .rule_config import RuleConfig

logger = logging.getLogger(__file__)


class Config:
    DEFAULT_FORBIDDEN_MANAGED_POLICY_ARNS = [
        "arn:aws:iam::aws:policy/AdministratorAccess",
        "arn:aws:iam::aws:policy/IAMFullAccess",
        "arn:aws:iam::aws:policy/job-function/NetworkAdministrator",
    ]
    DEFAULT_FORBIDDEN_RESOURCE_STAR_ACTION_PREFIXES = [
        # catch Action * Resource *
        "*",
        # stop S3 modifications on Resource *
        "s3:Put",
        "s3:Delete",
        # DynamoDB
        # http://docs.aws.amazon.com/IAM/latest/UserGuide/list_dynamodb.html
        "dynamodb:GetItem",
        "dynamodb:Delete",
        # IAM
        # http://docs.aws.amazon.com/IAM/latest/UserGuide/list_iam.html
        "iam:Add",
        "iam:Attach",
        "iam:Create",
        "iam:Delete",
        "iam:Put",
        "iam:Update",
        "iam:Remove",
        # pword / MFA STUFF
        "iam:ChangePassword",
        "iam:ResyncMFADevice",
        "iam:Deactivate",
        "iam:Enable",
        # EC2
        "ec2:DeleteCustomerGateway",
        "ec2:DeleteDhcpOptions",
        "ec2:DeleteFlowLogs",
        "ec2:DeleteInternetGateway",
        "ec2:DeleteNatGateway",
        # must keep as DeleteNetworkInterface needs to be allowed (for Lambda)
        "ec2:DeleteNetworkAcl",
        "ec2:DeleteNetworkAclEntry",
        "ec2:DeleteRoute",
        "ec2:DeleteRouteTable",
        "ec2:DeleteSecurityGroup",
        "ec2:DeleteSpotDatafeedSubscription",
        "ec2:DeleteSubnet",
        "ec2:DeleteVpc",
        "ec2:CreateSubnet",
        "ec2:CreateNatGateway",
        "ec2:CreateDhcpOptions",
        "ec2:CreateCustomerGateway",
        "ecs:*",
        # other lovely services
        "cloudtrail:",
        "aws-portal:",
        "acm:",
        "trustedadvisor:",
        "aws-marketplace",
        "directconnect:",
    ]

    def __init__(
        self,
        *,
        project_name=None,
        service_name=None,
        stack_name=None,
        rules=None,
        event=None,
        template_url=None,
        aws_region=None,
        aws_account_name=None,
        aws_account_id=None,
        aws_user_agent=None,
        aws_principals=None,
        aws_service_accounts=None,
        rules_config=None,
        rules_filters=None,
    ):
        self.project_name = project_name
        self.service_name = service_name
        self.stack_name = stack_name
        self.event = event
        self.rules = rules
        self.template_url = template_url
        self.aws_region = aws_region
        self.aws_account_name = aws_account_name
        self.aws_account_id = aws_account_id
        self.aws_user_agent = aws_user_agent
        if aws_service_accounts is None:
            self.aws_service_accounts = {
                "cloudtrail_account_ids": AWS_CLOUDTRAIL_ACCOUNT_IDS,
                "elasticache_backup_canonical_ids": AWS_ELASTICACHE_BACKUP_CANONICAL_IDS,
                "elb_logs_account_ids": AWS_ELB_LOGS_ACCOUNT_IDS,
                "redshift_audit_account_ids": AWS_REDSHIFT_AUDIT_ACCOUNT_IDS,
            }
        else:
            self.aws_service_accounts = aws_service_accounts

        self.forbidden_managed_policy_arns = list(self.DEFAULT_FORBIDDEN_MANAGED_POLICY_ARNS)

        self.forbidden_resource_star_action_prefixes = list(self.DEFAULT_FORBIDDEN_RESOURCE_STAR_ACTION_PREFIXES)

        # Set up a string list of allowed principals. If kept empty it will allow any AWS principal
        self.aws_principals = aws_principals if aws_principals is not None else []

        self.rules_config = rules_config if rules_config is not None else {}
        self.rules_filters: DefaultDict[str, List[Filter]] = defaultdict(list)
        if rules_filters:
            self.add_filters(rules_filters)

    def get_rule_config(self, rule_name: str) -> RuleConfig:
        rule_config = self.rules_config.get(rule_name)
        if rule_config is None:
            return RuleConfig()
        elif isinstance(rule_config, RuleConfig):
            return rule_config
        return RuleConfig(**rule_config)

    def get_rule_filters(self, rule_name: str) -> List[Filter]:
        return self.rules_filters.get(rule_name, [])

    def load_rules_config_file(self, rules_config_file: TextIOWrapper):
        filename = rules_config_file.name

        if not Path(filename).is_file():
            raise RuntimeError(f"{filename} doesn't exist")

        try:
            ext = Path(filename).suffix
            module_name = "__rules_config__"
            if ext not in [".py", ".pyc"]:
                raise RuntimeError("Configuration file should have a valid Python extension.")
            spec = importlib.util.spec_from_file_location(module_name, filename)
            module = importlib.util.module_from_spec(spec)
            sys.modules[module_name] = module
            spec.loader.exec_module(module)
            rules_config = vars(module).get("RULES_CONFIG")
            # Validate rules_config format
            RulesConfigMapping(__root__=rules_config)
            self.rules_config = rules_config
        except Exception:
            logger.exception(f"Failed to read config file: {filename}")
            raise

    def add_filters_from_dir(self, path: str):
        if not Path(path).is_dir():
            raise RuntimeError(f"{path} doesn't exist")

        try:
            module_name = "__rules_config__"
            filenames = sorted(itertools.chain(Path(path).glob("*.py"), Path(path).glob("*.pyc")))
            for filename in filenames:
                spec = importlib.util.spec_from_file_location(module_name, filename.absolute())
                module = importlib.util.module_from_spec(spec)
                sys.modules[module_name] = module
                spec.loader.exec_module(module)
                filters = vars(module).get("FILTERS")
                if not filters:
                    continue
                # Validate filters format
                RulesFiltersMapping(__root__=filters)
                self.add_filters(filters=filters)
                logger.debug(f"{filename} loaded")
        except Exception:
            logger.exception(f"Failed to read files in path: {path}")
            raise

    def add_filters(self, filters: List[Filter]):
        for filter in filters:
            for rule in filter.rules:
                self.rules_filters[rule].append(filter)


class RulesConfigMapping(BaseModel):
    __root__: Dict[str, RuleConfig]


class RulesFiltersMapping(BaseModel):
    __root__: List[Filter]



import logging
import re
from typing import Any, Callable, Dict, List, Optional, Set, Union

from pydantic import BaseModel, validator
from pydash.objects import get

from cfripper.model.enums import RuleMode, RuleRisk

VALID_FUNCTIONS = [
    "eq",
    "ne",
    "lt",
    "gt",
    "le",
    "ge",
    "not",
    "or",
    "and",
    "in",
    "regex",
    "regex:ignorecase",
    "exists",
    "empty",
    "ref",
]

logger = logging.getLogger(__file__)


def get_implemented_filter_function(function_name: str, debug: bool) -> Callable:
    def param_resolver(f):
        def wrap(*args, **kwargs):
            calculated_parameters = [arg(kwargs) for arg in args]
            result = f(*calculated_parameters, **kwargs)
            if debug:
                logger.debug(f"{function_name}({', '.join(str(x) for x in calculated_parameters)}) -> {result}")
            return result

        return wrap

    implemented_filter_functions = {
        "eq": param_resolver(lambda a, b, **kwargs: a == b),
        "ne": param_resolver(lambda a, b, **kwargs: a != b),
        "lt": param_resolver(lambda a, b, **kwargs: a < b),
        "gt": param_resolver(lambda a, b, **kwargs: a > b),
        "le": param_resolver(lambda a, b, **kwargs: a <= b),
        "ge": param_resolver(lambda a, b, **kwargs: a >= b),
        "not": param_resolver(lambda a, **kwargs: not a),
        "or": lambda *args, **kwargs: any(arg(kwargs) for arg in args),
        "and": lambda *args, **kwargs: all(arg(kwargs) for arg in args),
        "in": param_resolver(lambda a, b, **kwargs: a in b),
        "regex": param_resolver(lambda *args, **kwargs: bool(re.match(*args))),
        "regex:ignorecase": param_resolver(lambda *args, **kwargs: bool(re.match(*args, re.IGNORECASE))),
        "exists": param_resolver(lambda a, **kwargs: a is not None),
        "empty": param_resolver(lambda *args, **kwargs: len(args) == 0),
        "ref": param_resolver(lambda param_name, **kwargs: get(kwargs, param_name)),
    }
    return implemented_filter_functions[function_name]


def is_resolvable_dict(value: Any) -> bool:
    return isinstance(value, dict) and len(value) == 1 and next(iter(value)) in VALID_FUNCTIONS


def build_evaluator(tree: Union[str, int, float, bool, List, Dict], debug: bool = False) -> Callable:
    if is_resolvable_dict(tree):
        function_name, nodes = list(tree.items())[0]
        if not isinstance(nodes, list):
            nodes = [nodes]
        nodes = [build_evaluator(node, debug) for node in nodes]
        function_resolver = get_implemented_filter_function(function_name, debug)
        return lambda kwargs: function_resolver(*nodes, **kwargs)

    return lambda kwargs: tree


class Filter(BaseModel):
    debug: bool = False
    reason: str = ""
    eval: Union[Dict, Callable]
    rule_mode: Optional[RuleMode] = None
    risk_value: Optional[RuleRisk] = None
    rules: Set[str] = None

    @validator("eval", pre=True)
    def set_eval(cls, eval, values):
        return build_evaluator(eval, values["debug"])

    def __call__(self, **kwargs):
        if self.debug:
            logger.debug(f"Filter: {self.reason}")
        result = self.eval(kwargs)
        if self.debug:
            logger.debug(f"Filter result: {result}")
        return result



__all__ = ["HardcodedRDSPasswordRule"]

from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel
from pycfmodel.model.parameter import Parameter
from pycfmodel.model.resources.generic_resource import GenericResource

from cfripper.model.enums import RuleGranularity
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule


class HardcodedRDSPasswordRule(Rule):
    """
    Checks that any RDS clusters or instances aren't exposing their passwords.
    The rule forbids default password parameters and any missing `NoEcho` for RDS passwords.

    Risk:
        Not setting this correctly can lead to malicious agents attempting to gain access to your
        RDS instaces with a default password, or by reading the value that will be printed in plain
        text in the AWS console and logs if `NoEcho` is not set.

    Fix:
        When defining a password **do not use** the default value.
        If you specify a default password and you don’t provide a parameter, it will use the default
        which can be found clear text in the CloudFormation file.

    Code for fix:
        ````yml
        Parameters:
          MasterUserPassword:
            NoEcho: true
            Description: The database admin account password
            MinLength: 8
            Type: String

        ...

        Resources:
          RDSCluster:
            Type: AWS::RDS::DBCluster
            DeletionPolicy: "Snapshot"
            Properties:
              ...
              MasterUserPassword: !Ref 'MasterUserPassword'
              ...
        ````

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `Resource`                       | Resource that is being addressed                               |
    """

    REASON_DEFAULT = "Default RDS {} password parameter (readable in plain-text) for {}."
    REASON_MISSING_NOECHO = "RDS {} password parameter missing NoEcho for {}."
    GRANULARITY = RuleGranularity.RESOURCE

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        password_protected_cluster_ids = []
        instances_to_check = []

        for logical_id, resource in cfmodel.Resources.items():
            # flag insecure RDS Clusters.
            if resource.Type == "AWS::RDS::DBCluster":
                failure_added = self._failure_added(result, logical_id, resource, extras)
                if not failure_added:
                    password_protected_cluster_ids.append(logical_id)

            # keep track of RDS instances so they can be examined in the code below.
            elif resource.Type == "AWS::RDS::DBInstance":
                instances_to_check.append((logical_id, resource))

        # check each instance with the context of clusters.
        for logical_id, resource in instances_to_check:
            if resource.Properties.get("DBClusterIdentifier") and any(
                cluster_id in resource.Properties.get("DBClusterIdentifier")
                for cluster_id in password_protected_cluster_ids
            ):
                continue

            self._failure_added(result, logical_id, resource, extras)
        return result

    def _failure_added(
        self, result: Result, logical_id: str, resource: GenericResource, extras: Optional[Dict] = None
    ) -> bool:
        master_user_password = resource.Properties.get("MasterUserPassword", Parameter.NO_ECHO_NO_DEFAULT)
        resource_type = resource.Type.replace("AWS::RDS::DB", "")
        if master_user_password == Parameter.NO_ECHO_WITH_DEFAULT:
            self.add_failure_to_result(
                result,
                self.REASON_DEFAULT.format(resource_type, logical_id),
                resource_ids={logical_id},
                context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
            )
            return True
        elif master_user_password not in (Parameter.NO_ECHO_NO_DEFAULT, Parameter.NO_ECHO_WITH_VALUE):
            self.add_failure_to_result(
                result,
                self.REASON_MISSING_NOECHO.format(resource_type, logical_id),
                resource_ids={logical_id},
                context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
            )
            return True

        return False



__all__ = ["PrivilegeEscalationRule"]

from pycfmodel.model.resources.resource import Resource
from pycfmodel.model.resources.s3_bucket_policy import S3BucketPolicy

from cfripper.model.enums import RuleGranularity
from cfripper.rules.base_rules import BaseDangerousPolicyActions


class PrivilegeEscalationRule(BaseDangerousPolicyActions):
    f"""
    Checks for any dangerous IAM actions that could allow privilege escalation and potentially
    represent a large security risk.
    See [current blacklisted IAM actions]\
(https://github.com/Skyscanner/cfripper/blob/master/cfripper/rules/privilege_escalation.py#L25).

    Fix:
        Unless strictly necessary, do not use actions in the IAM action blacklist. CloudFormation files that do require
        these actions should be added to the allowlist.

    {BaseDangerousPolicyActions.DEFAULT_FILTERS_CONTEXT}
    """

    GRANULARITY = RuleGranularity.ACTION
    REASON = "{} has blacklisted IAM actions: {}"
    EXCLUDED_RESOURCE_TYPES = (S3BucketPolicy,)
    RESOURCE_TYPES = (Resource,)
    DANGEROUS_ACTIONS = [
        "iam:AddUserToGroup",
        "iam:AttachGroupPolicy",
        "iam:AttachRolePolicy",
        "iam:AttachUserPolicy",
        "iam:CreateAccessKey",
        "iam:CreateLoginProfile",
        "iam:CreatePolicy",
        "iam:CreatePolicyVersion",
        "iam:PutGroupPolicy",
        "iam:PutRolePolicy",
        "iam:PutUserPolicy",
        "iam:SetDefaultPolicyVersion",
        "iam:UpdateAssumeRolePolicy",
        "iam:UpdateLoginProfile",
    ]



__all__ = ["SNSTopicPolicyNotPrincipalRule", "SNSTopicDangerousPolicyActionsRule"]

from typing import Dict, Optional

from pycfmodel.model.resources.sns_topic_policy import SNSTopicPolicy

from cfripper.model.enums import RuleGranularity, RuleRisk
from cfripper.model.result import Result
from cfripper.rules.base_rules import BaseDangerousPolicyActions, ResourceSpecificRule


class SNSTopicPolicyNotPrincipalRule(ResourceSpecificRule):
    """
    Checks if an SNS topic policy has an Allow + a NotPrincipal.

    Risk:
        AWS **strongly** recommends against using `NotPrincipal` in the same policy statement as `"Effect": "Allow"`.
        Doing so grants the permissions specified in the policy statement to all principals except the one named
        in the `NotPrincipal` element. By doing this, you might grant access to anonymous (unauthenticated) users.

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `SNSTopicPolicy`                 | Resource that is being addressed                               |
        |`statement`              | `Statement`                      | Statement being checked found in the Resource                  |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "SNS Topic policy {} should not allow Allow and NotPrincipal at the same time"
    RESOURCE_TYPES = (SNSTopicPolicy,)

    def resource_invoke(self, resource: SNSTopicPolicy, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for statement in resource.Properties.PolicyDocument._statement_as_list():
            if statement.NotPrincipal:
                self.add_failure_to_result(
                    result,
                    self.REASON.format(logical_id),
                    resource_ids={logical_id},
                    context={
                        "config": self._config,
                        "extras": extras,
                        "logical_id": logical_id,
                        "resource": resource,
                        "statement": statement,
                    },
                )
        return result


class SNSTopicDangerousPolicyActionsRule(BaseDangerousPolicyActions):
    f"""
    Checks for dangerous permissions in Action statements in an SNS Topic Policy.

    Risk:
        This is deemed a potential security risk as it could allow privilege escalation.

    {BaseDangerousPolicyActions.DEFAULT_FILTERS_CONTEXT}
    """

    REASON = "SNS Topic policy {} should not not include the following dangerous actions: {}"
    RISK_VALUE = RuleRisk.MEDIUM
    RESOURCE_TYPES = (SNSTopicPolicy,)

    DANGEROUS_ACTIONS = [
        "sns:AddPermission",
        "sns:RemovePermission",
        "sns:TagResource",
        "sns:UntagResource",
    ]



import logging
from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel

from cfripper.config.config import Config
from cfripper.model.enums import RuleMode
from cfripper.model.result import Result

logger = logging.getLogger(__file__)


class RuleProcessor:
    def __init__(self, *args):
        self.rules = args

    def process_cf_template(self, cfmodel: CFModel, config: Config, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for rule in self.rules:
            if rule.rule_mode == RuleMode.DISABLED:
                continue

            try:
                result += rule.invoke(cfmodel, extras)
            except Exception as other_exception:
                result.add_exception(other_exception)
                logger.exception(
                    "{} crashed with {} for project - {}, service - {}, stack - {}".format(
                        type(rule).__name__,
                        type(other_exception).__name__,
                        config.project_name,
                        config.service_name,
                        config.stack_name,
                    )
                )
                continue
        return result



AWS_ELB_LOGS_ACCOUNT_IDS = [
    # From https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html
    "009996457667",  # Elastic Load Balancing Account ID - Europe (Paris)
    "027434742980",  # Elastic Load Balancing Account ID - US West (N. California)
    "033677994240",  # Elastic Load Balancing Account ID - US East (Ohio)
    "037604701340",  # Elastic Load Balancing Account ID - China (Ningxia)
    "048591011584",  # Elastic Load Balancing Account ID - AWS GovCloud (US-West)
    "054676820928",  # Elastic Load Balancing Account ID - Europe (Frankfurt)
    "076674570225",  # Elastic Load Balancing Account ID - Middle East (Bahrain)
    "098369216593",  # Elastic Load Balancing Account ID - Africa (Cape Town)
    "114774131450",  # Elastic Load Balancing Account ID - Asia Pacific (Singapore)
    "127311923021",  # Elastic Load Balancing Account ID - US East (N. Virginia)
    "156460612806",  # Elastic Load Balancing Account ID - Europe (Ireland)
    "190560391635",  # Elastic Load Balancing Account ID - AWS GovCloud (US-East)
    "383597477331",  # Elastic Load Balancing Account ID - Asia Pacific (Osaka)
    "507241528517",  # Elastic Load Balancing Account ID - South America (São Paulo)
    "582318560864",  # Elastic Load Balancing Account ID - Asia Pacific (Tokyo)
    "600734575887",  # Elastic Load Balancing Account ID - Asia Pacific (Seoul)
    "635631232127",  # Elastic Load Balancing Account ID - Europe (Milan)
    "638102146993",  # Elastic Load Balancing Account ID - China (Beijing)
    "652711504416",  # Elastic Load Balancing Account ID - Europe (London)
    "718504428378",  # Elastic Load Balancing Account ID - Asia Pacific (Mumbai)
    "754344448648",  # Elastic Load Balancing Account ID - Asia Pacific (Hong Kong)
    "783225319266",  # Elastic Load Balancing Account ID - Asia Pacific (Sydney)
    "797873946194",  # Elastic Load Balancing Account ID - US West (Oregon)
    "897822967062",  # Elastic Load Balancing Account ID - Europe (Stockholm)
    "985666609251",  # Elastic Load Balancing Account ID - Canada (Central)
]

AWS_CLOUDTRAIL_ACCOUNT_IDS = [
    # From https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-supported-regions.html
    "034638983726",  # CloudTrail Account ID - Middle East (Bahrain)
    "035351147821",  # CloudTrail Account ID - Europe (Frankfurt)
    "086441151436",  # CloudTrail Account ID - US East (N. Virginia)
    "113285607260",  # CloudTrail Account ID - US West (Oregon)
    "119688915426",  # CloudTrail Account ID - Asia Pacific (Hong Kong)
    "193415116832",  # CloudTrail Account ID - China (Beijing)
    "216624486486",  # CloudTrail Account ID - Asia Pacific (Tokyo)
    "262312530599",  # CloudTrail Account ID - Europe (Paris)
    "282025262664",  # CloudTrail Account ID - Europe (London)
    "284668455005",  # CloudTrail Account ID - Asia Pacific (Sydney)
    "388731089494",  # CloudTrail Account ID - US West (N. California)
    "475085895292",  # CloudTrail Account ID - US East (Ohio)
    "492519147666",  # CloudTrail Account ID - Asia Pacific (Seoul)
    "525921808201",  # CloudTrail Account ID - Africa (Cape Town)
    "608710470296",  # CloudTrail Account ID - AWS GovCloud (US-West)
    "669305197877",  # CloudTrail Account ID - Europe (Milan)
    "681348832753",  # CloudTrail Account ID - China (Ningxia)
    "765225791966",  # CloudTrail Account ID - Asia Pacific (Osaka)
    "814480443879",  # CloudTrail Account ID - South America (São Paulo)
    "819402241893",  # CloudTrail Account ID - Canada (Central)
    "829690693026",  # CloudTrail Account ID - Europe (Stockholm)
    "859597730677",  # CloudTrail Account ID - Europe (Ireland)
    "886388586500",  # CloudTrail Account ID - AWS GovCloud (US-East)
    "903692715234",  # CloudTrail Account ID - Asia Pacific (Singapore)
    "977081816279",  # CloudTrail Account ID - Asia Pacific (Mumbai)
]

AWS_REDSHIFT_AUDIT_ACCOUNT_IDS = [
    # From https://docs.aws.amazon.com/redshift/latest/mgmt/db-auditing.html#rs-db-auditing-cloud-trail-rs-acct-ids
    "041313461515",  # Redshift Audit Logs Account ID - Europe (Milan) Region
    "051362938876",  # Redshift Audit Logs Account ID - Middle East (Bahrain) Region
    "246478207311",  # Redshift Audit Logs Account ID - Europe (Ireland) Region
    "368064434614",  # Redshift Audit Logs Account ID - US East (N. Virginia) Region
    "392442076723",  # Redshift Audit Logs Account ID - South America (São Paulo) Region
    "398671365691",  # Redshift Audit Logs Account ID - Asia Pacific (Osaka) Region
    "408097707231",  # Redshift Audit Logs Account ID - Asia Pacific (Mumbai) Region
    "420376844563",  # Redshift Audit Logs Account ID - Africa (Cape Town) Region
    "434091160558",  # Redshift Audit Logs Account ID - Europe (Frankfurt) Region
    "473191095985",  # Redshift Audit Logs Account ID - US West (Oregon) Region
    "485979073181",  # Redshift Audit Logs Account ID - Asia Pacific (Sydney) Region
    "553461782468",  # Redshift Audit Logs Account ID - Europe (Stockholm) Region
    "615915377779",  # Redshift Audit Logs Account ID - Asia Pacific (Tokyo) Region
    "651179539253",  # Redshift Audit Logs Account ID - Asia Pacific (Hong Kong) Region
    "694668203235",  # Redshift Audit Logs Account ID - Europe (Paris) Region
    "703715109447",  # Redshift Audit Logs Account ID - US West (N. California) Region
    "713597048934",  # Redshift Audit Logs Account ID - Asia Pacific (Seoul) Region
    "764870610256",  # Redshift Audit Logs Account ID - Canada (Central) Region
    "790247189693",  # Redshift Audit Logs Account ID - US East (Ohio) Region
    "885798887673",  # Redshift Audit Logs Account ID - Europe (London) Region
    "960118270566",  # Redshift Audit Logs Account ID - Asia Pacific (Singapore) Region
]


AWS_ELASTICACHE_BACKUP_CANONICAL_IDS = [
    # From https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html
    "b14d6a125bdf69854ed8ef2e71d8a20b7c490f252229b806e514966e490b8d83",  # China (Beijing) and China (Ningxia) Regions
    "40fa568277ad703bd160f66ae4f83fc9dfdfd06c2f1b5060ca22442ac3ef8be6",  # AWS GovCloud (US-West) Region
    "540804c33a284a299d2547575ce1010f2312ef3da9b3a053c8bc45bf233e4353",  # All other AWS Regions
]



import re

"""
Check for Principals where root is being used.
Valid:
- arn:aws:iam::437628376:root
- arn:aws:iam::344345345:root
- arn:aws:iam:::root
Invalid:
- arn:aws:iam::437628376:not-root
- potato
"""
REGEX_CROSS_ACCOUNT_ROOT = re.compile(r"arn:aws:iam::\d*:root")

"""
Check for use of wildcard in two bad cases: full wildcard, or wildcard in account ID.
Valid:
- *
- arn:aws:iam::*:12345
Invalid:
- arn:aws:iam::444455556666:root
- potato
- arn:aws:iam::12345:*
"""
REGEX_FULL_WILDCARD_PRINCIPAL = re.compile(r"^((\w*:){0,1}\*|arn:aws:iam::\*:.*)$")

"""
Check for use of wildcard or account-wide principals.
Valid:
- arn:aws:iam::123456789012:*
- arn:aws:iam::123456789012:service-*
- arn:aws:iam::123456789012:root
- 123456789012
Invalid:
- *
- potato
- arn:aws:iam::123456789012:*not-root
"""
REGEX_PARTIAL_WILDCARD_PRINCIPAL = re.compile(r"^\d{12}|arn:aws:iam::.*:(.*\*|root)$")

"""
Check for use of wildcard, when applied to the specific elements of an Action.
For example, sts:AssumeRole* or sts:*. This regex is not checking for use of `*` on its own.
Valid:
- sts:AssumeRole*
- sts:*
Invalid:
- sts:AssumeRole
- sts:AssumeRole-Thing-This
- *
"""
REGEX_WILDCARD_POLICY_ACTION = re.compile(r"^(\w*:)(\w*)\*(\w*)$")

"""
Check for Principals where root is being used.
Valid:
- *
- abc*def
- abcdef*
- *abcdef
Invalid:
- arn:aws:iam::437628376:not-root
- potato
"""
REGEX_CONTAINS_STAR = re.compile(r"^.*[*].*$")


"""
Check for root wildcard
Valid:
- *
Invalid:
- abc*def
- abcdef*
- *abcdef
- arn:aws:iam::437628376:not-root
- potato
"""
REGEX_IS_STAR = re.compile(r"^\*$")


"""
Check for arns
It has 4 groups. The first one for service name, the second one for region, the third, for account id, the last one
for resource id
Valid:
- arn:aws:iam::437628376:not-root
- arn:aws:iam::437628376:root
- arn:aws:s3:::my_corporate_bucket
Invalid:
- potato
"""
REGEX_ARN = re.compile(r"^arn:aws:(\w+):(\w*):(\d*):(.+)$")


"""
Check for arns that allow the full aws service range in a particular service
Valid:
- arn:aws:s3:::*
- arn:aws:iam:*:*:*
- arn:aws:*:::*
Invalid:
- potato
- arn:aws:s3:::my_corporate_bucket
- arn:aws:iam::437628376:root
"""
REGEX_WILDCARD_ARN = re.compile(r"^arn:aws:([*\w]+):(\*)?:(\*)?:([*?]+)$")


"""
Check for iam arns
It has 2 groups. The first one for account id, the last one for resource id
Valid:
- arn:aws:iam::437628376:not-root
Invalid:
- arn:aws:s3:::my_corporate_bucket
- potato
"""
REGEX_IAM_ARN = re.compile(r"^arn:aws:iam::(\d*):(.*)$")


"""
Check for sts arns
It has 2 groups. The first one for account id, the last one for resource id
Valid:
- arn:aws:sts::437628376:not-root
Invalid:
- arn:aws:s3:::my_corporate_bucket
- potato
"""
REGEX_STS_ARN = re.compile(r"^arn:aws:sts::(\d*):(.*)$")


"""
Check for wildcards after colons
Valid:
- *
- arn:aws:iam::437628376:*
- sns:*
Invalid:
- arn:aws:s3:::my_corporate_bucket
- arn:aws:s3:::my_corporate_bucket*
- arn:aws:s3:::*my_corporate_bucket
- potato
- sns:Get*
"""
REGEX_HAS_STAR_OR_STAR_AFTER_COLON = re.compile(r"^(\w*:)*\*$")



from cfripper.config.rule_config import RuleConfig
from cfripper.config.rule_configs.allow_http_ports_open_to_world import (
    allow_http_ports_open_to_world_rules_config_filter,
)
from cfripper.config.rule_configs.firehose_ips import firehose_ips_rules_config_filter
from cfripper.model.enums import RuleMode

RULES_CONFIG = {
    "EC2SecurityGroupMissingEgressRule": RuleConfig(rule_mode=RuleMode.DISABLED),
}

FILTERS = [allow_http_ports_open_to_world_rules_config_filter, firehose_ips_rules_config_filter]



from cfripper.config.filter import Filter
from cfripper.model.enums import RuleMode

"""
To use this Filter, or any Filter, make sure to include it in the `Config` instantiation.

```python
FILTERS = [firehose_ips_rules_config_filter]

config = Config(
    ...
    rules_filters=FILTERS,
)
```
"""

# Adapted from https://docs.aws.amazon.com/firehose/latest/dev/controlling-access.html
FIREHOSE_IPS = {
    "13.113.196.224/27",  # Asia Pacific (Tokyo)
    "13.209.1.64/27",  # Asia Pacific (Seoul)
    "13.210.67.224/27",  # Asia Pacific (Sydney)
    "13.228.64.192/27",  # Asia Pacific (Singapore)
    "13.232.67.32/27",  # Asia Pacific (Mumbai)
    "13.244.121.224/277",  # Africa (Cape Town)
    "13.53.63.224/27",  # Europe (Stockholm)
    "13.57.135.192/27",  # US West (N. California)
    "13.58.135.96/27",  # US East (Ohio)
    "15.161.135.128/27",  # Europe (Milan)
    "15.185.91.0/27",  # Middle East (Bahrain)
    "161.189.23.64/27",  # China (Ningxia)
    "18.130.1.96/27",  # Europe (London)
    "18.162.221.32/27",  # Asia Pacific (Hong Kong)
    "18.228.1.128/27",  # South America (São Paulo)
    "18.253.138.96/27",  # AWS GovCloud (US-East)
    "35.158.127.160/27",  # Europe (Frankfurt)
    "35.180.1.96/27",  # Europe (Paris)
    "35.183.92.128/27",  # Canada (Central)
    "52.19.239.192/27",  # Europe (Ireland)
    "52.61.204.160/27",  # AWS GovCloud (US-West)
    "52.70.63.192/27",  # US East (N. Virginia)
    "52.81.151.32/27",  # China (Beijing)
    "52.89.255.224/27",  # US West (Oregon)
}

firehose_ips_rules_config_filter = Filter(
    reason=(
        "Exclude Kinesis Data Firehose IPs to allow access from Amazon Redshift Clusters. "
        "See https://docs.aws.amazon.com/firehose/latest/dev/controlling-access.html"
    ),
    rule_mode=RuleMode.ALLOWED,
    eval={"and": [{"exists": {"ref": "ingress_ip"}}, {"in": [{"ref": "ingress_ip"}, FIREHOSE_IPS]}]},
    rules={"EC2SecurityGroupOpenToWorldRule"},
)



__all__ = ["S3BucketPublicReadAclAndListStatementRule", "S3BucketPublicReadWriteAclRule", "S3BucketPublicReadAclRule"]

import logging
import re
from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel
from pycfmodel.model.resources.s3_bucket import S3Bucket
from pycfmodel.model.resources.s3_bucket_policy import S3BucketPolicy

from cfripper.model.enums import RuleGranularity, RuleRisk
from cfripper.model.result import Result
from cfripper.rules.base_rules import ResourceSpecificRule, Rule

logger = logging.getLogger(__file__)


class S3BucketPublicReadAclAndListStatementRule(Rule):
    # TODO: refactor regex to regex file.
    """
    Checks if any S3 bucket policy has a public read ACL and `List` permission in the bucket policy.

    Fix:
        Unless the bucket is hosting static content and needs to be accessed publicly,
        these bucket policies should be locked down.

    Filters context:
        | Parameter     | Type               | Description                                                    |
        |:-------------:|:------------------:|:--------------------------------------------------------------:|
        |`config`       | str                | `config` variable available inside the rule                    |
        |`extras`       | str                | `extras` variable available inside the rule                    |
        |`logical_id`   | str                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`     | `S3BucketPolicy`   | Resource that is being addressed                               |
        |`bucket_name`  | str                | Name of the S3 bucket being analysed                           |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "S3 Bucket {} should not have a public read acl and list bucket statement"

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if isinstance(resource, S3BucketPolicy) and resource.Properties.PolicyDocument.allowed_actions_with(
                re.compile(r"^s3:L.*$")
            ):
                bucket_name = resource.Properties.Bucket
                if not isinstance(bucket_name, str):
                    logger.warning(f"Not adding {type(self).__name__} failure in {logical_id} – try resolving?")
                    continue
                if "UNDEFINED_PARAM_" in bucket_name:
                    bucket_name = bucket_name[len("UNDEFINED_PARAM_") :]

                bucket = cfmodel.Resources.get(bucket_name)
                if bucket and bucket.Properties.AccessControl == "PublicRead":
                    self.add_failure_to_result(
                        result,
                        self.REASON.format(logical_id),
                        resource_ids={logical_id},
                        context={
                            "config": self._config,
                            "extras": extras,
                            "logical_id": logical_id,
                            "resource": resource,
                            "bucket_name": bucket_name,
                        },
                    )
        return result


class S3BucketPublicReadWriteAclRule(ResourceSpecificRule):
    """
    Checks if any S3 bucket policy has access control set to `PublicReadWrite`.

    Risk:
        Unless required, S3 buckets should not have Public Write available on a bucket. This allows anyone
        to write any objects to your S3 bucket.

    Fix:
        Remove any configuration that looks like `"AccessControl": "PublicReadWrite"` from your S3 bucket policy.

    Filters context:
        | Parameter     | Type               | Description                                                    |
        |:-------------:|:------------------:|:--------------------------------------------------------------:|
        |`config`       | str                | `config` variable available inside the rule                    |
        |`extras`       | str                | `extras` variable available inside the rule                    |
        |`logical_id`   | str                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`     | `S3Bucket`         | S3 Bucket that is being addressed                              |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "S3 Bucket {} should not have a public read-write acl"
    RESOURCE_TYPES = (S3Bucket,)
    RISK_VALUE = RuleRisk.HIGH

    def resource_invoke(self, resource: S3Bucket, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()
        if resource.Properties.AccessControl == "PublicReadWrite":
            self.add_failure_to_result(
                result,
                self.REASON.format(logical_id),
                resource_ids={logical_id},
                context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
            )
        return result


class S3BucketPublicReadAclRule(ResourceSpecificRule):
    """
    Checks if any S3 bucket policy has access control set to `PublicRead`.

    Risk:
        Unless the bucket is hosting static content, S3 buckets should not have Public Read available on a bucket.
        This allows anyone to read any objects to your S3 bucket.

    Fix:
        Remove any configuration that looks like `"AccessControl": "PublicRead"` from your S3 bucket policy.

    Filters context:
        | Parameter     | Type               | Description                                                    |
        |:-------------:|:------------------:|:--------------------------------------------------------------:|
        |`config`       | str                | `config` variable available inside the rule                    |
        |`extras`       | str                | `extras` variable available inside the rule                    |
        |`logical_id`   | str                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`     | `S3Bucket`         | S3 Bucket that is being addressed                              |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "S3 Bucket {} should not have a public-read acl"
    RESOURCE_TYPES = (S3Bucket,)
    RISK_VALUE = RuleRisk.HIGH

    def resource_invoke(self, resource: S3Bucket, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()
        if resource.Properties.AccessControl == "PublicRead":
            self.add_failure_to_result(
                result,
                self.REASON.format(logical_id),
                resource_ids={logical_id},
                context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
            )
        return result



__all__ = ["EBSVolumeHasSSERule"]

from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel

from cfripper.model.enums import RuleGranularity
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule


class EBSVolumeHasSSERule(Rule):
    """
    Checks that server side encryption is enabled for all EBS volumes.

    Risk:
        Data that is not encrypted at rest could breach regulatory compliance
        and allow easier access for an attacker to view any instace storage data
        of your EC2 instance.

    Fix:
        Enable server-side encryption on EBS volumes.

    Code for fix:
        ````json
        {
            "Type" : "AWS::EC2::Volume",
            "Properties" : {
                ...
                "Encrypted" : true,
                ...
            }
        }
        ````

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `Resource`                       | EC2 Volume that is being addressed                             |
    """

    REASON = "EBS volume {} should have server-side encryption enabled"
    GRANULARITY = RuleGranularity.RESOURCE

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if resource.Type == "AWS::EC2::Volume":
                if resource.Properties.get("Encrypted") != "true":
                    self.add_failure_to_result(
                        result,
                        self.REASON.format(logical_id),
                        resource_ids={logical_id},
                        context={
                            "config": self._config,
                            "extras": extras,
                            "logical_id": logical_id,
                            "resource": resource,
                        },
                    )
        return result



__all__ = [
    "GenericWildcardPolicyRule",
    "S3BucketPolicyWildcardActionRule",
    "SNSTopicPolicyWildcardActionRule",
    "SQSQueuePolicyWildcardActionRule",
]
import logging
from typing import Dict, Optional, Type

from pycfmodel.model.cf_model import CFModel
from pycfmodel.model.resources.resource import Resource
from pycfmodel.model.resources.s3_bucket_policy import S3BucketPolicy
from pycfmodel.model.resources.sns_topic_policy import SNSTopicPolicy
from pycfmodel.model.resources.sqs_queue_policy import SQSQueuePolicy

from cfripper.config.regex import REGEX_HAS_STAR_OR_STAR_AFTER_COLON
from cfripper.model.enums import RuleGranularity
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule

logger = logging.getLogger(__file__)


class GenericWildcardPolicyRule(Rule):
    """
    Abstract rule that checks for use of the wildcard `*` character in Actions of Policy Documents of AWS Resources.
    This rule must be inherited by another class to be used, with `AWS_RESOURCE` set to the resource to be checked.
    See `S3BucketPolicyWildcardActionRule` and `SQSQueuePolicyWildcardActionRule` for examples.
    """

    REASON = "The {} {} should not allow a `*` action"

    GRANULARITY = RuleGranularity.RESOURCE

    AWS_RESOURCE: Type[Resource] = None

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        if self.AWS_RESOURCE is None:
            logger.warning(f"Not running {type(self).__name__} rule as AWS_RESOURCE is not defined.")
        else:
            for logical_id, resource in cfmodel.Resources.items():
                if isinstance(resource, self.AWS_RESOURCE) and resource.Properties.PolicyDocument.allowed_actions_with(
                    REGEX_HAS_STAR_OR_STAR_AFTER_COLON
                ):
                    self.add_failure_to_result(
                        result,
                        self.REASON.format(self.AWS_RESOURCE.__name__, logical_id),
                        resource_ids={logical_id},
                        context={
                            "config": self._config,
                            "extras": extras,
                            "logical_id": logical_id,
                            "resource": resource,
                        },
                    )
        return result


class S3BucketPolicyWildcardActionRule(GenericWildcardPolicyRule):
    """
    Checks for use of the wildcard `*` character in the Actions of Policy Documents of S3 Bucket Policies.
    This rule is a subclass of `GenericWildcardPolicyRule`.

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `S3BucketPolicy`                 | Resource that is being addressed                               |
    """

    AWS_RESOURCE = S3BucketPolicy


class SNSTopicPolicyWildcardActionRule(GenericWildcardPolicyRule):
    """
    Checks for use of the wildcard `*` character in the Actions of Policy Documents of SQS Queue Policies.
    This rule is a subclass of `GenericWildcardPolicyRule`.

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `SNSTopicPolicy`                 | Resource that is being addressed                               |
    """

    AWS_RESOURCE = SNSTopicPolicy


class SQSQueuePolicyWildcardActionRule(GenericWildcardPolicyRule):
    """
    Checks for use of the wildcard `*` character in the Actions of Policy Documents of SQS Queue Policies.
    This rule is a subclass of `GenericWildcardPolicyRule`.

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `SQSQueuePolicy`                 | Resource that is being addressed                               |
    """

    AWS_RESOURCE = SQSQueuePolicy



__all__ = [
    "WildcardResourceRule",
]
import logging
from typing import Dict, Optional

from pycfmodel.model.resources.generic_resource import GenericResource
from pycfmodel.model.resources.iam_role import IAMRole
from pycfmodel.model.resources.kms_key import KMSKey
from pycfmodel.model.resources.properties.policy_document import PolicyDocument
from pycfmodel.model.resources.properties.statement import Statement
from pycfmodel.model.resources.resource import Resource

from cfripper.cloudformation_actions_only_accepts_wildcard import CLOUDFORMATION_ACTIONS_ONLY_ACCEPTS_WILDCARD
from cfripper.config.regex import REGEX_IS_STAR, REGEX_WILDCARD_ARN
from cfripper.model.enums import RuleGranularity, RuleMode
from cfripper.model.result import Result
from cfripper.rules.base_rules import ResourceSpecificRule

logger = logging.getLogger(__file__)


class WildcardResourceRule(ResourceSpecificRule):
    """
    Generic rule that detects actions that accept a resource and are using a wildcard.

    Risk:
        Give roles access to undesired resources.

    Fix:
        Check AWS docs to use the recommended resource value.

    Filters context:
        | Parameter    | Type             | Description                                                     |
        |:------------:|:----------------:|:---------------------------------------------------------------:|
        |`config`      | str              | `config` variable available inside the rule                     |
        |`extras`      | str              | `extras` variable available inside the rule                     |
        |`logical_id`  | str              | ID used in Cloudformation to refer the resource being analysed  |
        |`policy_name` | `Optional[str]`  | If available, the policy name                                   |
        |`statement`   | `Statement`      | Statement being checked found in the Resource                   |
        |`action`      | `Optional[str]`  | Action that has a wildcard resource. If None, means all actions |
    """

    RESOURCE_TYPES = (Resource,)
    REASON_WITH_POLICY_NAME = '"{}" is using a wildcard resource in "{}" for "{}"'
    REASON_WITHOUT_POLICY_NAME = '"{}" is using a wildcard resource for "{}"'
    REASON_ALL_ACTIONS_WITH_POLICY_NAME = '"{}" is using a wildcard resource in "{}" allowing all actions'
    REASON_ALL_ACTIONS_WITHOUT_POLICY_NAME = '"{}" is using a wildcard resource allowing all actions'

    def resource_invoke(self, resource: Resource, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()

        for policy in resource.policy_documents:
            self._check_policy_document(result, logical_id, policy.policy_document, policy.name, extras)
        if isinstance(resource, IAMRole):
            self._check_policy_document(result, logical_id, resource.Properties.AssumeRolePolicyDocument, None, extras)
        elif isinstance(resource, KMSKey):
            self._check_policy_document(result, logical_id, resource.Properties.KeyPolicy, None, extras)
        elif isinstance(resource, GenericResource):
            if hasattr(resource, "Properties"):
                policy_document = resource.Properties.get("PolicyDocument")
                if policy_document:
                    self._check_policy_document(result, logical_id, PolicyDocument(**policy_document), None, extras)

        return result

    def _check_policy_document(
        self, result: Result, logical_id: str, policy_document: PolicyDocument, policy_name: Optional[str], extras: Dict
    ):
        statements_to_review = policy_document.statements_with(REGEX_IS_STAR) + policy_document.statements_with(
            REGEX_WILDCARD_ARN
        )
        for statement in statements_to_review:
            self._check_statement(result, logical_id, policy_name, statement, extras=extras)

    def _check_statement(
        self, result: Result, logical_id: str, policy_name: Optional[str], statement: Statement, extras: Dict
    ):
        if statement.Effect and statement.Effect == "Deny":
            return

        if statement.actions_with(REGEX_IS_STAR):
            if statement.Condition:
                self._add_to_result(result, logical_id, policy_name, None, statement, extras, monitor=True)
            else:
                self._add_to_result(result, logical_id, policy_name, None, statement, extras)
        else:
            for action in statement.get_expanded_action_list():
                if action in CLOUDFORMATION_ACTIONS_ONLY_ACCEPTS_WILDCARD:
                    logger.info(f"Action {action} only accepts wildcard, ignoring...")
                elif action.lower().startswith("kms:"):
                    # When KMS Key policies use * in the resource, that * will only apply this policy to the KMS Key being created
                    # so we must not flag this
                    # Source: https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html
                    logger.info(f"KMS Action {action} only accepts wildcard, ignoring...")
                elif statement.Condition:
                    self._add_to_result(result, logical_id, policy_name, action, statement, extras, monitor=True)
                else:
                    self._add_to_result(result, logical_id, policy_name, action, statement, extras)

    def _add_to_result(
        self,
        result: Result,
        logical_id: str,
        policy_name: Optional[str],
        action: Optional[str],
        statement: Statement,
        extras: Dict,
        monitor: bool = False,
    ):
        self.add_failure_to_result(
            result=result,
            reason=self._build_reason(logical_id, action, policy_name),
            granularity=RuleGranularity.ACTION,
            resource_ids={logical_id},
            actions=set(statement.get_action_list()),
            rule_mode=RuleMode.MONITOR if monitor else None,
            context={
                "config": self._config,
                "extras": extras,
                "logical_id": logical_id,
                "policy_name": policy_name,
                "statement": statement,
                "action": action,
            },
        )

    def _build_reason(self, logical_id: str, action: Optional[str], policy_name: Optional[str]) -> str:
        if action:
            if policy_name:
                return self.REASON_WITH_POLICY_NAME.format(logical_id, policy_name, action)
            else:
                return self.REASON_WITHOUT_POLICY_NAME.format(logical_id, action)
        else:
            if policy_name:
                return self.REASON_ALL_ACTIONS_WITH_POLICY_NAME.format(logical_id, policy_name)
            else:
                return self.REASON_ALL_ACTIONS_WITHOUT_POLICY_NAME.format(logical_id)



from typing import Collection, List, Optional

from pydantic import BaseModel, Extra

from cfripper.model.enums import RuleGranularity, RuleMode, RuleRisk


class Failure(BaseModel):
    granularity: RuleGranularity
    reason: str
    risk_value: RuleRisk
    rule: str
    rule_mode: RuleMode
    actions: Optional[set] = set()
    resource_ids: Optional[set] = set()

    class Config(BaseModel.Config):
        extra = Extra.forbid

    def serializable(self):
        return {
            "rule": self.rule,
            "reason": self.reason,
            "rule_mode": self.rule_mode,
            "risk_value": self.risk_value,
            "resource_ids": sorted(self.resource_ids or []),
            "actions": sorted(self.actions or []),
            "granularity": self.granularity,
        }


class Result(BaseModel):
    class Config(BaseModel.Config):
        extra = Extra.forbid
        arbitrary_types_allowed = True

    exceptions: List[Exception] = []
    failures: List[Failure] = []

    def get_failures(
        self,
        include_rule_modes: Optional[Collection[RuleMode]] = None,
        exclude_rule_modes: Optional[Collection[RuleMode]] = None,
    ) -> List[Failure]:
        result = []
        for failure in self.failures:
            if (not exclude_rule_modes or failure.rule_mode not in exclude_rule_modes) and (
                not include_rule_modes or failure.rule_mode in include_rule_modes
            ):
                result.append(failure)
        return result

    def add_exception(self, ex: Exception):
        self.exceptions.append(ex)

    def add_failure(
        self,
        rule: str,
        reason: str,
        rule_mode: RuleMode,
        risk_value: RuleRisk,
        granularity: RuleGranularity,
        resource_ids=None,
        actions=None,
    ):
        self.failures.append(
            Failure(
                rule=rule,
                reason=reason,
                rule_mode=rule_mode,
                risk_value=risk_value,
                resource_ids=resource_ids,
                actions=actions,
                granularity=granularity,
            )
        )

    @property
    def valid(self) -> bool:
        return not any(failure for failure in self.failures if failure.rule_mode == RuleMode.BLOCKING)

    def __add__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented

        return Result(exceptions=self.exceptions + other.exceptions, failures=self.failures + other.failures)



from pluggy import PluginManager

from cfripper import rules
from cfripper.config.pluggy import hookspec


def setup_plugin_manager():
    pm = PluginManager("cfripper")
    pm.add_hookspecs(hookspec)
    return pm


def load_all_plugins(plugin_manager: PluginManager):
    plugin_manager.load_setuptools_entrypoints("cfripper")  # Dynamically load plugins using entry points
    plugin_manager.register(rules)  # Register default rules


def get_all_rules():
    plugin_manager = setup_plugin_manager()
    load_all_plugins(plugin_manager)
    plugins_output = plugin_manager.hook.cfripper_get_rules()
    rules = {k: v for d in plugins_output for k, v in d.items()}
    return rules



from cfripper.config.pluggy import hookimpl
from cfripper.rules.base_rules import BaseDangerousPolicyActions, PrincipalCheckingRule, ResourceSpecificRule
from cfripper.rules.cloudformation_authentication import CloudFormationAuthenticationRule
from cfripper.rules.cross_account_trust import (
    CrossAccountCheckingRule,
    CrossAccountTrustRule,
    KMSKeyCrossAccountTrustRule,
    S3CrossAccountTrustRule,
)
from cfripper.rules.ebs_volume_has_sse import EBSVolumeHasSSERule
from cfripper.rules.ec2_security_group import (
    EC2SecurityGroupIngressOpenToWorldRule,
    EC2SecurityGroupMissingEgressRule,
    EC2SecurityGroupOpenToWorldRule,
)
from cfripper.rules.hardcoded_RDS_password import HardcodedRDSPasswordRule
from cfripper.rules.iam_roles import IAMRolesOverprivilegedRule, IAMRoleWildcardActionOnPolicyRule
from cfripper.rules.kms_key_rotation_enabled import KMSKeyEnabledKeyRotation
from cfripper.rules.kms_key_wildcard_principal import KMSKeyWildcardPrincipalRule
from cfripper.rules.managed_policy_on_user import ManagedPolicyOnUserRule
from cfripper.rules.policy_on_user import PolicyOnUserRule
from cfripper.rules.privilege_escalation import PrivilegeEscalationRule
from cfripper.rules.s3_bucket_policy import S3BucketPolicyPrincipalRule
from cfripper.rules.s3_lifecycle_configuration import S3LifecycleConfigurationRule
from cfripper.rules.s3_object_versioning import S3ObjectVersioningRule
from cfripper.rules.s3_public_access import (
    S3BucketPublicReadAclAndListStatementRule,
    S3BucketPublicReadAclRule,
    S3BucketPublicReadWriteAclRule,
)
from cfripper.rules.sns_topic_policy import SNSTopicDangerousPolicyActionsRule, SNSTopicPolicyNotPrincipalRule
from cfripper.rules.sqs_queue_policy import (
    SQSDangerousPolicyActionsRule,
    SQSQueuePolicyNotPrincipalRule,
    SQSQueuePolicyPublicRule,
)
from cfripper.rules.wildcard_policies import (
    S3BucketPolicyWildcardActionRule,
    SNSTopicPolicyWildcardActionRule,
    SQSQueuePolicyWildcardActionRule,
)
from cfripper.rules.wildcard_principals import FullWildcardPrincipalRule, PartialWildcardPrincipalRule
from cfripper.rules.wildcard_resource_rule import WildcardResourceRule

DEFAULT_RULES = {
    rule.__name__: rule
    for rule in (
        CloudFormationAuthenticationRule,
        CrossAccountTrustRule,
        EBSVolumeHasSSERule,
        EC2SecurityGroupIngressOpenToWorldRule,
        EC2SecurityGroupMissingEgressRule,
        EC2SecurityGroupOpenToWorldRule,
        FullWildcardPrincipalRule,
        HardcodedRDSPasswordRule,
        IAMRolesOverprivilegedRule,
        IAMRoleWildcardActionOnPolicyRule,
        KMSKeyCrossAccountTrustRule,
        KMSKeyWildcardPrincipalRule,
        KMSKeyEnabledKeyRotation,
        ManagedPolicyOnUserRule,
        PartialWildcardPrincipalRule,
        PolicyOnUserRule,
        PrivilegeEscalationRule,
        S3BucketPolicyPrincipalRule,
        S3LifecycleConfigurationRule,
        S3BucketPolicyWildcardActionRule,
        S3BucketPublicReadAclAndListStatementRule,
        S3BucketPublicReadWriteAclRule,
        S3BucketPublicReadAclRule,
        S3CrossAccountTrustRule,
        S3ObjectVersioningRule,
        SNSTopicDangerousPolicyActionsRule,
        SNSTopicPolicyNotPrincipalRule,
        SNSTopicPolicyWildcardActionRule,
        SQSDangerousPolicyActionsRule,
        SQSQueuePolicyNotPrincipalRule,
        SQSQueuePolicyPublicRule,
        SQSQueuePolicyWildcardActionRule,
        WildcardResourceRule,
    )
}

BASE_CLASSES = {
    rule.__name__: rule
    for rule in (BaseDangerousPolicyActions, CrossAccountCheckingRule, PrincipalCheckingRule, ResourceSpecificRule,)
}


@hookimpl
def cfripper_get_rules():
    return DEFAULT_RULES



__all__ = ["SQSQueuePolicyNotPrincipalRule", "SQSQueuePolicyPublicRule", "SQSDangerousPolicyActionsRule"]

import logging
from typing import Dict, Optional

from pycfmodel.model.resources.sqs_queue_policy import SQSQueuePolicy

from cfripper.config.regex import REGEX_HAS_STAR_OR_STAR_AFTER_COLON
from cfripper.model.enums import RuleGranularity, RuleRisk
from cfripper.model.result import Result
from cfripper.rules.base_rules import BaseDangerousPolicyActions, ResourceSpecificRule

logger = logging.getLogger(__file__)


class SQSQueuePolicyNotPrincipalRule(ResourceSpecificRule):
    """
    Checks if an SQS Queue policy has an Allow + a NotPrincipal.

    Risk:
        AWS **strongly** recommends against using `NotPrincipal` in the same policy statement as `"Effect": "Allow"`.
        Doing so grants the permissions specified in the policy statement to all principals except the one named
        in the `NotPrincipal` element. By doing this, you might grant access to anonymous (unauthenticated) users.

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `SQSQueuePolicy`                 | Resource that is being addressed                               |
        |`statement`              | `Statement`                      | Statement being checked found in the Resource                  |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    RESOURCE_TYPES = (SQSQueuePolicy,)
    REASON = "SQS Queue policy {} should not allow Allow and NotPrincipal at the same time"

    def resource_invoke(self, resource: SQSQueuePolicy, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for statement in resource.Properties.PolicyDocument._statement_as_list():
            if statement.NotPrincipal:
                self.add_failure_to_result(
                    result,
                    self.REASON.format(logical_id),
                    resource_ids={logical_id},
                    context={
                        "config": self._config,
                        "extras": extras,
                        "logical_id": logical_id,
                        "resource": resource,
                        "statement": statement,
                    },
                )
        return result


class SQSQueuePolicyPublicRule(ResourceSpecificRule):
    """
    Checks for wildcard principals in Allow statements in an SQS Queue Policy.

    Risk:
        This is deemed a potential security risk as anyone would be able to interact with your queue.

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `SQSQueuePolicy`                 | Resource that is being addressed                               |
        |`statement`              | `Statement`                      | Statement being checked found in the Resource                  |
    """

    REASON = "SQS Queue policy {} should not be public"
    RISK_VALUE = RuleRisk.HIGH
    GRANULARITY = RuleGranularity.RESOURCE
    RESOURCE_TYPES = (SQSQueuePolicy,)

    def resource_invoke(self, resource: SQSQueuePolicy, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()
        if resource.Properties.PolicyDocument.allowed_principals_with(REGEX_HAS_STAR_OR_STAR_AFTER_COLON):
            for statement in resource.Properties.PolicyDocument._statement_as_list():
                if statement.Effect == "Allow" and statement.principals_with(REGEX_HAS_STAR_OR_STAR_AFTER_COLON):
                    if statement.Condition and statement.Condition.dict():
                        logger.warning(
                            f"Not adding {type(self).__name__} failure in {logical_id} "
                            f"because there are conditions: {statement.Condition}"
                        )
                    else:
                        self.add_failure_to_result(
                            result,
                            self.REASON.format(logical_id),
                            resource_ids={logical_id},
                            context={
                                "config": self._config,
                                "extras": extras,
                                "logical_id": logical_id,
                                "resource": resource,
                                "statement": statement,
                            },
                        )
        return result


class SQSDangerousPolicyActionsRule(BaseDangerousPolicyActions):
    f"""
    Checks for dangerous permissions in Action statements in an SQS Queue Policy.

    Risk:
        This is deemed a potential security risk as it'd allow various attacks to the queue.

    {BaseDangerousPolicyActions.DEFAULT_FILTERS_CONTEXT}
    """

    REASON = "SQS Queue policy {} should not not include the following dangerous actions: {}"
    RISK_VALUE = RuleRisk.MEDIUM
    RESOURCE_TYPES = (SQSQueuePolicy,)

    DANGEROUS_ACTIONS = [
        "sqs:AddPermission",
        "sqs:CreateQueue",
        "sqs:DeleteQueue",
        "sqs:RemovePermission",
        "sqs:TagQueue",
        "sqs:UnTagQueue",
    ]



from typing import Optional

from pydantic import BaseModel

from cfripper.model.enums import RuleMode, RuleRisk


class RuleConfig(BaseModel):
    rule_mode: Optional[RuleMode] = None
    risk_value: Optional[RuleRisk] = None



CLOUDFORMATION_ACTIONS_ONLY_ACCEPTS_WILDCARD = [
    "batch:CancelJob",
    "batch:DescribeComputeEnvironments",
    "batch:DescribeJobDefinitions",
    "batch:DescribeJobQueues",
    "batch:DescribeJobs",
    "batch:ListJobs",
    "batch:TerminateJob",
    "cloudformation:CreateStackSet",
    "cloudformation:CreateUploadBucket",
    "cloudformation:DeregisterType",
    "cloudformation:DescribeAccountLimits",
    "cloudformation:DescribeStackDriftDetectionStatus",
    "cloudformation:DescribeType",
    "cloudformation:DescribeTypeRegistration",
    "cloudformation:EstimateTemplateCost",
    "cloudformation:ListExports",
    "cloudformation:ListImports",
    "cloudformation:ListStacks",
    "cloudformation:ListTypeRegistrations",
    "cloudformation:ListTypeVersions",
    "cloudformation:ListTypes",
    "cloudformation:RegisterType",
    "cloudformation:SetTypeDefaultVersion",
    "cloudformation:ValidateTemplate",
    "cloudwatch:DeleteAnomalyDetector",
    "cloudwatch:DescribeAlarmsForMetric",
    "cloudwatch:DescribeAnomalyDetectors",
    "cloudwatch:DescribeInsightRules",
    "cloudwatch:GetMetricData",
    "cloudwatch:GetMetricStatistics",
    "cloudwatch:GetMetricWidgetImage",
    "cloudwatch:ListDashboards",
    "cloudwatch:ListMetrics",
    "cloudwatch:PutAnomalyDetector",
    "cloudwatch:PutMetricData",
    "dynamodb:ListTables",
    "ec2:AcceptReservedInstancesExchangeQuote",
    "ec2:AdvertiseByoipCidr",
    "ec2:AllocateAddress",
    "ec2:AssignIpv6Addresses",
    "ec2:AssignPrivateIpAddresses",
    "ec2:AssociateAddress",
    "ec2:AssociateDhcpOptions",
    "ec2:AssociateRouteTable",
    "ec2:AssociateSubnetCidrBlock",
    "ec2:AssociateVpcCidrBlock",
    "ec2:AttachInternetGateway",
    "ec2:AttachNetworkInterface",
    "ec2:AttachVpnGateway",
    "ec2:BundleInstance",
    "ec2:CancelBundleTask",
    "ec2:CancelConversionTask",
    "ec2:CancelExportTask",
    "ec2:CancelImportTask",
    "ec2:CancelReservedInstancesListing",
    "ec2:CancelSpotFleetRequests",
    "ec2:CancelSpotInstanceRequests",
    "ec2:ConfirmProductInstance",
    "ec2:CopyFpgaImage",
    "ec2:CopyImage",
    "ec2:CreateCustomerGateway",
    "ec2:CreateDefaultSubnet",
    "ec2:CreateDefaultVpc",
    "ec2:CreateImage",
    "ec2:CreateNetworkInterface",  # It shouldn't be here AWS Support Case: 7344470691
    "ec2:CreateNetworkAclEntry",
    "ec2:CreateReservedInstancesListing",
    "ec2:CreateRouteTable",
    "ec2:CreateSpotDatafeedSubscription",
    "ec2:CreateVpcEndpointConnectionNotification",
    "ec2:CreateVpnConnectionRoute",
    "ec2:CreateVpnGateway",
    "ec2:DeleteEgressOnlyInternetGateway",
    "ec2:DeleteFleets",
    "ec2:DeleteFpgaImage",
    "ec2:DeleteKeyPair",
    "ec2:DeleteNatGateway",
    "ec2:DeleteNetworkInterface",
    "ec2:DeleteNetworkInterfacePermission",
    "ec2:DeletePlacementGroup",
    "ec2:DeleteSpotDatafeedSubscription",
    "ec2:DeleteSubnet",
    "ec2:DeleteVpc",
    "ec2:DeleteVpcEndpointConnectionNotifications",
    "ec2:DeleteVpnConnection",
    "ec2:DeleteVpnConnectionRoute",
    "ec2:DeleteVpnGateway",
    "ec2:DeprovisionByoipCidr",
    "ec2:DeregisterImage",
    "ec2:DeregisterInstanceEventNotificationAttributes",
    "ec2:DescribeAccountAttributes",
    "ec2:DescribeAddresses",
    "ec2:DescribeAggregateIdFormat",
    "ec2:DescribeAvailabilityZones",
    "ec2:DescribeBundleTasks",
    "ec2:DescribeByoipCidrs",
    "ec2:DescribeCapacityReservations",
    "ec2:DescribeClassicLinkInstances",
    "ec2:DescribeClientVpnAuthorizationRules",
    "ec2:DescribeClientVpnConnections",
    "ec2:DescribeClientVpnEndpoints",
    "ec2:DescribeClientVpnRoutes",
    "ec2:DescribeClientVpnTargetNetworks",
    "ec2:DescribeCoipPools",
    "ec2:DescribeConversionTasks",
    "ec2:DescribeCustomerGateways",
    "ec2:DescribeDhcpOptions",
    "ec2:DescribeEgressOnlyInternetGateways",
    "ec2:DescribeElasticGpus",
    "ec2:DescribeExportImageTasks",
    "ec2:DescribeExportTasks",
    "ec2:DescribeFastSnapshotRestores",
    "ec2:DescribeFleetHistory",
    "ec2:DescribeFleetInstances",
    "ec2:DescribeFleets",
    "ec2:DescribeFlowLogs",
    "ec2:DescribeFpgaImageAttribute",
    "ec2:DescribeFpgaImages",
    "ec2:DescribeHostReservationOfferings",
    "ec2:DescribeHostReservations",
    "ec2:DescribeHosts",
    "ec2:DescribeIamInstanceProfileAssociations",
    "ec2:DescribeIdFormat",
    "ec2:DescribeIdentityIdFormat",
    "ec2:DescribeImageAttribute",
    "ec2:DescribeImages",
    "ec2:DescribeImportImageTasks",
    "ec2:DescribeImportSnapshotTasks",
    "ec2:DescribeInstanceAttribute",
    "ec2:DescribeInstanceCreditSpecifications",
    "ec2:DescribeInstanceEventNotificationAttributes",
    "ec2:DescribeInstanceStatus",
    "ec2:DescribeInstanceTypeOfferings",
    "ec2:DescribeInstanceTypes",
    "ec2:DescribeInstances",
    "ec2:DescribeInternetGateways",
    "ec2:DescribeKeyPairs",
    "ec2:DescribeLaunchTemplateVersions",
    "ec2:DescribeLaunchTemplates",
    "ec2:DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociations",
    "ec2:DescribeLocalGatewayRouteTableVpcAssociations",
    "ec2:DescribeLocalGatewayRouteTables",
    "ec2:DescribeLocalGatewayVirtualInterfaceGroups",
    "ec2:DescribeLocalGatewayVirtualInterfaces",
    "ec2:DescribeLocalGateways",
    "ec2:DescribeManagedPrefixLists",
    "ec2:DescribeMovingAddresses",
    "ec2:DescribeNatGateways",
    "ec2:DescribeNetworkAcls",
    "ec2:DescribeNetworkInterfaceAttribute",
    "ec2:DescribeNetworkInterfacePermissions",
    "ec2:DescribeNetworkInterfaces",
    "ec2:DescribePlacementGroups",
    "ec2:DescribePrefixLists",
    "ec2:DescribePrincipalIdFormat",
    "ec2:DescribePublicIpv4Pools",
    "ec2:DescribeRegions",
    "ec2:DescribeReservedInstances",
    "ec2:DescribeReservedInstancesListings",
    "ec2:DescribeReservedInstancesModifications",
    "ec2:DescribeReservedInstancesOfferings",
    "ec2:DescribeRouteTables",
    "ec2:DescribeScheduledInstanceAvailability",
    "ec2:DescribeScheduledInstances",
    "ec2:DescribeSecurityGroupReferences",
    "ec2:DescribeSecurityGroups",
    "ec2:DescribeSnapshotAttribute",
    "ec2:DescribeSnapshots",
    "ec2:DescribeSpotDatafeedSubscription",
    "ec2:DescribeSpotFleetInstances",
    "ec2:DescribeSpotFleetRequestHistory",
    "ec2:DescribeSpotFleetRequests",
    "ec2:DescribeSpotInstanceRequests",
    "ec2:DescribeSpotPriceHistory",
    "ec2:DescribeStaleSecurityGroups",
    "ec2:DescribeSubnets",
    "ec2:DescribeTags",
    "ec2:DescribeTrafficMirrorFilters",
    "ec2:DescribeTrafficMirrorSessions",
    "ec2:DescribeTrafficMirrorTargets",
    "ec2:DescribeTransitGatewayAttachments",
    "ec2:DescribeTransitGatewayMulticastDomains",
    "ec2:DescribeTransitGatewayPeeringAttachments",
    "ec2:DescribeTransitGatewayRouteTables",
    "ec2:DescribeTransitGatewayVpcAttachments",
    "ec2:DescribeTransitGateways",
    "ec2:DescribeVolumeAttribute",
    "ec2:DescribeVolumeStatus",
    "ec2:DescribeVolumes",
    "ec2:DescribeVolumesModifications",
    "ec2:DescribeVpcAttribute",
    "ec2:DescribeVpcClassicLink",
    "ec2:DescribeVpcClassicLinkDnsSupport",
    "ec2:DescribeVpcEndpointConnectionNotifications",
    "ec2:DescribeVpcEndpointConnections",
    "ec2:DescribeVpcEndpointServiceConfigurations",
    "ec2:DescribeVpcEndpointServicePermissions",
    "ec2:DescribeVpcEndpointServices",
    "ec2:DescribeVpcEndpoints",
    "ec2:DescribeVpcPeeringConnections",
    "ec2:DescribeVpcs",
    "ec2:DescribeVpnConnections",
    "ec2:DescribeVpnGateways",
    "ec2:DetachInternetGateway",
    "ec2:DetachNetworkInterface",
    "ec2:DetachVpnGateway",
    "ec2:DisableEbsEncryptionByDefault",
    "ec2:DisableVgwRoutePropagation",
    "ec2:DisableVpcClassicLinkDnsSupport",
    "ec2:DisassociateAddress",
    "ec2:DisassociateRouteTable",
    "ec2:DisassociateSubnetCidrBlock",
    "ec2:DisassociateVpcCidrBlock",
    "ec2:EnableEbsEncryptionByDefault",
    "ec2:EnableVgwRoutePropagation",
    "ec2:EnableVolumeIO",
    "ec2:EnableVpcClassicLinkDnsSupport",
    "ec2:ExportClientVpnClientCertificateRevocationList",
    "ec2:ExportClientVpnClientConfiguration",
    "ec2:ExportImage",
    "ec2:ExportTransitGatewayRoutes",
    "ec2:GetCapacityReservationUsage",
    "ec2:GetCoipPoolUsage",
    "ec2:GetConsoleOutput",
    "ec2:GetDefaultCreditSpecification",
    "ec2:GetEbsDefaultKmsKeyId",
    "ec2:GetEbsEncryptionByDefault",
    "ec2:GetHostReservationPurchasePreview",
    "ec2:GetLaunchTemplateData",
    "ec2:GetManagedPrefixListAssociations",
    "ec2:GetManagedPrefixListEntries",
    "ec2:GetPasswordData",
    "ec2:GetReservedInstancesExchangeQuote",
    "ec2:GetTransitGatewayAttachmentPropagations",
    "ec2:GetTransitGatewayMulticastDomainAssociations",
    "ec2:GetTransitGatewayRouteTableAssociations",
    "ec2:GetTransitGatewayRouteTablePropagations",
    "ec2:ImportImage",
    "ec2:ImportInstance",
    "ec2:ImportKeyPair",
    "ec2:ImportSnapshot",
    "ec2:ImportVolume",
    "ec2:ModifyDefaultCreditSpecification",
    "ec2:ModifyEbsDefaultKmsKeyId",
    "ec2:ModifyFleet",
    "ec2:ModifyFpgaImageAttribute",
    "ec2:ModifyHosts",
    "ec2:ModifyIdFormat",
    "ec2:ModifyIdentityIdFormat",
    "ec2:ModifyImageAttribute",
    "ec2:ModifyInstanceAttribute",
    "ec2:ModifyInstanceCapacityReservationAttributes",
    "ec2:ModifyInstanceMetadataOptions",
    "ec2:ModifyInstancePlacement",
    "ec2:ModifyNetworkInterfaceAttribute",
    "ec2:ModifyReservedInstances",
    "ec2:ModifySpotFleetRequest",
    "ec2:ModifySubnetAttribute",
    "ec2:ModifyVolume",
    "ec2:ModifyVolumeAttribute",
    "ec2:ModifyVpcAttribute",
    "ec2:ModifyVpcEndpointConnectionNotification",
    "ec2:ModifyVpcPeeringConnectionOptions",
    "ec2:ModifyVpcTenancy",
    "ec2:ModifyVpnTunnelCertificate",
    "ec2:MonitorInstances",
    "ec2:MoveAddressToVpc",
    "ec2:ProvisionByoipCidr",
    "ec2:PurchaseHostReservation",
    "ec2:PurchaseReservedInstancesOffering",
    "ec2:PurchaseScheduledInstances",
    "ec2:RegisterImage",
    "ec2:RegisterInstanceEventNotificationAttributes",
    "ec2:ReleaseAddress",
    "ec2:ReleaseHosts",
    "ec2:ReplaceNetworkAclAssociation",
    "ec2:ReplaceNetworkAclEntry",
    "ec2:ReplaceRouteTableAssociation",
    "ec2:ReportInstanceStatus",
    "ec2:RequestSpotFleet",
    "ec2:RequestSpotInstances",
    "ec2:ResetEbsDefaultKmsKeyId",
    "ec2:ResetFpgaImageAttribute",
    "ec2:ResetImageAttribute",
    "ec2:ResetInstanceAttribute",
    "ec2:ResetNetworkInterfaceAttribute",
    "ec2:ResetSnapshotAttribute",
    "ec2:RestoreAddressToClassic",
    "ec2:RunScheduledInstances",
    "ec2:SearchLocalGatewayRoutes",
    "ec2:SearchTransitGatewayMulticastGroups",
    "ec2:SearchTransitGatewayRoutes",
    "ec2:UnassignIpv6Addresses",
    "ec2:UnassignPrivateIpAddresses",
    "ec2:UnmonitorInstances",
    "ec2:WithdrawByoipCidr",
    "ecr:GetAuthorizationToken",
    "ecs:CreateCapacityProvider",
    "ecs:CreateCluster",
    "ecs:CreateTaskSet",
    "ecs:DeleteAccountSetting",
    "ecs:DeregisterTaskDefinition",
    "ecs:DescribeTaskDefinition",
    "ecs:DiscoverPollEndpoint",
    "ecs:ListAccountSettings",
    "ecs:ListClusters",
    "ecs:ListServices",
    "ecs:ListTaskDefinitionFamilies",
    "ecs:ListTaskDefinitions",
    "ecs:PutAccountSetting",
    "ecs:PutAccountSettingDefault",
    "eks:CreateCluster",
    "eks:DescribeAddonVersions",
    "eks:ListClusters",
    "elasticbeanstalk:CheckDNSAvailability",
    "elasticbeanstalk:CreateStorageLocation",
    "elasticbeanstalk:DescribeAccountAttributes",
    "elasticbeanstalk:ListPlatformBranches",
    "elasticache:DescribeCacheEngineVersions",
    "elasticache:DescribeEngineDefaultParameters",
    "elasticache:DescribeEvents",
    "elasticache:DescribeReservedCacheNodesOfferings",
    "elasticache:DescribeServiceUpdates",
    "elasticache:RevokeCacheSecurityGroupIngress",
    "elasticloadbalancing:DescribeAccountLimits",
    "elasticloadbalancing:DescribeListenerCertificates",
    "elasticloadbalancing:DescribeListeners",
    "elasticloadbalancing:DescribeLoadBalancerAttributes",
    "elasticloadbalancing:DescribeLoadBalancerPolicies",
    "elasticloadbalancing:DescribeLoadBalancerPolicyTypes",
    "elasticloadbalancing:DescribeLoadBalancers",
    "elasticloadbalancing:DescribeRules",
    "elasticloadbalancing:DescribeSSLPolicies",
    "elasticloadbalancing:DescribeTags",
    "elasticloadbalancing:DescribeTargetGroupAttributes",
    "elasticloadbalancing:DescribeTargetGroups",
    "elasticloadbalancing:DescribeTargetHealth",
    "elasticloadbalancing:SetWebAcl",
    "es:AcceptInboundCrossClusterSearchConnection",
    "es:CreateElasticsearchServiceRole",
    "es:DeleteElasticsearchServiceRole",
    "es:DeleteInboundCrossClusterSearchConnection",
    "es:DeleteOutboundCrossClusterSearchConnection",
    "es:DescribeElasticsearchInstanceTypeLimits",
    "es:DescribeInboundCrossClusterSearchConnections",
    "es:DescribeOutboundCrossClusterSearchConnections",
    "es:DescribeReservedElasticsearchInstanceOfferings",
    "es:DescribeReservedElasticsearchInstances",
    "es:ListDomainNames",
    "es:ListElasticsearchInstanceTypeDetails",
    "es:ListElasticsearchInstanceTypes",
    "es:ListElasticsearchVersions",
    "es:PurchaseReservedElasticsearchInstanceOffering",
    "es:RejectInboundCrossClusterSearchConnection",
    "events:ListEventBuses",
    "events:ListEventSources",
    "events:ListPartnerEventSources",
    "events:ListRuleNamesByTarget",
    "events:ListRules",
    "events:PutPartnerEvents",
    "events:PutPermission",
    "events:RemovePermission",
    "events:TestEventPattern",
    "events:UntagResource",
    "iam:CreateAccountAlias",
    "iam:DeleteAccountAlias",
    "iam:DeleteAccountPasswordPolicy",
    "iam:GenerateCredentialReport",
    "iam:GenerateServiceLastAccessedDetails",
    "iam:GetAccountAuthorizationDetails",
    "iam:GetAccountPasswordPolicy",
    "iam:GetAccountSummary",
    "iam:GetContextKeysForCustomPolicy",
    "iam:GetCredentialReport",
    "iam:GetOrganizationsAccessReport",
    "iam:GetServiceLastAccessedDetails",
    "iam:GetServiceLastAccessedDetailsWithEntities",
    "iam:ListAccountAliases",
    "iam:ListGroups",
    "iam:ListOpenIDConnectProviders",
    "iam:ListPolicies",
    "iam:ListPoliciesGrantingServiceAccess",
    "iam:ListRoles",
    "iam:ListSAMLProviders",
    "iam:ListServerCertificates",
    "iam:ListUsers",
    "iam:ListVirtualMFADevices",
    "iam:SetSecurityTokenServicePreferences",
    "iam:SimulateCustomPolicy",
    "iam:UpdateAccountPasswordPolicy",
    "iot:CreateCertificateFromCsr",
    "iot:CreateKeysAndCertificate",
    "iot:CreatePolicy",
    "iot:DeleteRegistrationCode",
    "iot:DescribeEndpoint",
    "iot:GetLoggingOptions",
    "iot:GetRegistrationCode",
    "iot:ListCACertificates",
    "iot:ListCertificates",
    "iot:ListCertificatesByCA",
    "iot:ListOutgoingCertificates",
    "iot:ListPolicies",
    "iot:ListThingTypes",
    "iot:ListThings",
    "iot:ListTopicRules",
    "iot:RegisterCACertificate",
    "iot:RegisterCertificate",
    "lambda:CreateEventSourceMapping",
    "lambda:GetAccountSettings",
    "lambda:ListEventSourceMappings",
    "lambda:ListFunctions",
    "lambda:ListLayerVersions",
    "lambda:ListLayers",
    "rds:CancelExportTask",
    "rds:CreateDBProxy",
    "rds:DescribeAccountAttributes",
    "rds:DescribeCertificates",
    "rds:DescribeDBClusterEndpoints",
    "rds:DescribeDBClusterSnapshots",
    "rds:DescribeDBInstanceAutomatedBackups",
    "rds:DescribeDBInstances",
    "rds:DescribeEngineDefaultClusterParameters",
    "rds:DescribeEngineDefaultParameters",
    "rds:DescribeEventCategories",
    "rds:DescribeEvents",
    "rds:DescribeExportTasks",
    "rds:DescribeGlobalClusters",
    "rds:DescribeOrderableDBInstanceOptions",
    "rds:DescribeReservedDBInstancesOfferings",
    "rds:DescribeSourceRegions",
    "rds:StartExportTask",
    "route53:CreateHostedZone",
    "route53:CreateReusableDelegationSet",
    "route53:CreateTrafficPolicy",
    "route53:CreateTrafficPolicyInstance",
    "route53:CreateTrafficPolicyVersion",
    "route53:CreateVPCAssociationAuthorization",
    "route53:DeleteHostedZone",
    "route53:DeleteReusableDelegationSet",
    "route53:DeleteTrafficPolicy",
    "route53:DeleteTrafficPolicyInstance",
    "route53:DeleteVPCAssociationAuthorization",
    "route53:GetAccountLimit",
    "route53:GetChange",
    "route53:GetCheckerIpRanges",
    "route53:GetHealthCheckCount",
    "route53:GetHostedZone",
    "route53:GetHostedZoneCount",
    "route53:GetHostedZoneLimit",
    "route53:GetReusableDelegationSet",
    "route53:GetReusableDelegationSetLimit",
    "route53:GetTrafficPolicy",
    "route53:GetTrafficPolicyInstance",
    "route53:GetTrafficPolicyInstanceCount",
    "route53:ListHealthChecks",
    "route53:ListHostedZones",
    "route53:ListHostedZonesByName",
    "route53:ListReusableDelegationSets",
    "route53:ListTrafficPolicies",
    "route53:ListTrafficPolicyInstances",
    "route53:ListTrafficPolicyInstancesByHostedZone",
    "route53:ListTrafficPolicyInstancesByPolicy",
    "route53:ListTrafficPolicyVersions",
    "route53:UpdateHostedZoneComment",
    "route53:UpdateTrafficPolicyComment",
    "route53:UpdateTrafficPolicyInstance",
    "route53domains:CheckDomainAvailability",
    "route53domains:DeleteTagsForDomain",
    "route53domains:DisableDomainAutoRenew",
    "route53domains:DisableDomainTransferLock",
    "route53domains:EnableDomainAutoRenew",
    "route53domains:EnableDomainTransferLock",
    "route53domains:GetContactReachabilityStatus",
    "route53domains:GetDomainDetail",
    "route53domains:GetDomainSuggestions",
    "route53domains:GetOperationDetail",
    "route53domains:ListDomains",
    "route53domains:ListOperations",
    "route53domains:ListTagsForDomain",
    "route53domains:RegisterDomain",
    "route53domains:RenewDomain",
    "route53domains:ResendContactReachabilityEmail",
    "route53domains:RetrieveDomainAuthCode",
    "route53domains:TransferDomain",
    "route53domains:UpdateDomainContact",
    "route53domains:UpdateDomainContactPrivacy",
    "route53domains:UpdateDomainNameservers",
    "route53domains:UpdateTagsForDomain",
    "route53domains:ViewBilling",
    "s3:CreateJob",
    "s3:GetAccessPoint",
    "s3:GetAccountPublicAccessBlock",
    "s3:ListAccessPoints",
    "s3:ListAllMyBuckets",
    "s3:ListJobs",
    "s3:PutAccountPublicAccessBlock",
    "sns:CheckIfPhoneNumberIsOptedOut",
    "sns:CreatePlatformApplication",
    "sns:CreatePlatformEndpoint",
    "sns:DeleteEndpoint",
    "sns:DeletePlatformApplication",
    "sns:GetEndpointAttributes",
    "sns:GetPlatformApplicationAttributes",
    "sns:GetSMSAttributes",
    "sns:GetSubscriptionAttributes",
    "sns:ListEndpointsByPlatformApplication",
    "sns:ListPhoneNumbersOptedOut",
    "sns:ListPlatformApplications",
    "sns:ListSubscriptions",
    "sns:ListTopics",
    "sns:OptInPhoneNumber",
    "sns:SetEndpointAttributes",
    "sns:SetPlatformApplicationAttributes",
    "sns:SetSMSAttributes",
    "sns:SetSubscriptionAttributes",
    "sns:Unsubscribe",
    "xray:BatchGetTraces",
    "xray:GetEncryptionConfig",
    "xray:GetGroups",
    "xray:GetInsight",
    "xray:GetInsightEvents",
    "xray:GetInsightImpactGraph",
    "xray:GetInsightSummaries",
    "xray:GetSamplingRules",
    "xray:GetSamplingStatisticSummaries",
    "xray:GetSamplingTargets",
    "xray:GetServiceGraph",
    "xray:GetTimeSeriesServiceStatistics",
    "xray:GetTraceGraph",
    "xray:GetTraceSummaries",
    "xray:PutEncryptionConfig",
    "xray:PutTelemetryRecords",
    "xray:PutTraceSegments",
    "xray:PutTraceSegments",
    "xray:PutTraceSegments",
]



__all__ = ["KMSKeyEnabledKeyRotation"]

import logging
from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel

from cfripper.model.enums import RuleGranularity, RuleRisk
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule

logger = logging.getLogger(__file__)


class KMSKeyEnabledKeyRotation(Rule):
    """
    Check if EnableKeyRotation is true for symmetric KMS keys in principals in KMS Policies.

    Fix:
        Set EnableKeyRotation to true for any symmetric KMS key.

    Filters context:
        | Parameter           | Type                | Description                                                   |
        |:-------------------:|:------------------:|:--------------------------------------------------------------:|
        |`config`             | str                | `config` variable available inside the rule                    |
        |`extras`             | str                | `extras` variable available inside the rule                    |
        |`logical_id`         | str                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`           | `KMSKey`           | Resource that is being addressed                               |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "KMS Key {} should have the key rotation enabled for symmetric keys"
    RISK_VALUE = RuleRisk.HIGH

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.resources_filtered_by_type("AWS::KMS::Key").items():
            if not resource.Properties.KeySpec or resource.Properties.KeySpec == "SYMMETRIC_DEFAULT":
                if not resource.Properties.EnableKeyRotation or resource.Properties.EnableKeyRotation is False:
                    self.add_failure_to_result(
                        result,
                        self.REASON.format(logical_id),
                        resource_ids={logical_id},
                        context={
                            "config": self._config,
                            "extras": extras,
                            "logical_id": logical_id,
                            "resource": resource,
                        },
                    )
        return result



from itertools import chain, combinations

from cfripper.config.filter import Filter
from cfripper.model.enums import RuleMode

"""
To use this Filter, or any Filter, make sure to include it in the `Config` instantiation.

```python
FILTERS = [allow_http_ports_open_to_world_rules_config_filter]

config = Config(
    ...
    rules_filters=FILTERS,
)
```
"""


ALLOWED_PORTS = [80, 443]


def powerset(iterable):
    # https://docs.python.org/3/library/itertools.html#itertools-recipes
    # powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)
    s = list(iterable)
    return chain.from_iterable(combinations(s, r) for r in range(len(s) + 1))


allow_http_ports_open_to_world_rules_config_filter = Filter(
    reason="It can be acceptable to have Security Groups publicly available on ports 80 or 443.",
    rule_mode=RuleMode.ALLOWED,
    eval={
        "and": [
            {"exists": {"ref": "open_ports"}},
            {
                "or": [
                    {"eq": [list(subset_allowed_ports), {"ref": "open_ports"}]}
                    for subset_allowed_ports in powerset(ALLOWED_PORTS)
                    if subset_allowed_ports
                ]
            },
        ]
    },
    rules={"EC2SecurityGroupOpenToWorldRule", "EC2SecurityGroupIngressOpenToWorldRule"},
)



import logging
import os


def setup_logging():
    if not os.environ.get("AWS_ACCOUNT"):
        logging.basicConfig(level=logging.DEBUG)
        return logging

    logging.basicConfig(level=logging.WARN)
    return logging



__all__ = [
    "CrossAccountCheckingRule",
    "CrossAccountTrustRule",
    "KMSKeyCrossAccountTrustRule",
    "S3CrossAccountTrustRule",
]

import logging
from abc import ABC
from typing import Dict, Optional, Set

from pycfmodel.model.cf_model import CFModel
from pycfmodel.model.resources.iam_role import IAMRole
from pycfmodel.model.resources.kms_key import KMSKey
from pycfmodel.model.resources.properties.statement import Statement
from pycfmodel.model.resources.resource import Resource
from pycfmodel.model.resources.s3_bucket_policy import S3BucketPolicy

from cfripper.model.enums import RuleGranularity, RuleMode
from cfripper.model.result import Result
from cfripper.model.utils import get_account_id_from_principal
from cfripper.rules.base_rules import PrincipalCheckingRule

logger = logging.getLogger(__file__)


class CrossAccountCheckingRule(PrincipalCheckingRule, ABC):
    """
    Base class not intended to be instantiated, but inherited from.
    This class provides common methods used to detect access permissions from other accounts.
    """

    GRANULARITY = RuleGranularity.RESOURCE
    RESOURCE_TYPE: Resource
    PROPERTY_WITH_POLICYDOCUMENT: str

    @property
    def valid_principals(self) -> Set[str]:
        if self._valid_principals is None:
            self._valid_principals = self._get_allowed_from_config()
            if self._config.aws_account_id:
                self._valid_principals.add(self._config.aws_account_id)
        return self._valid_principals

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if isinstance(resource, self.RESOURCE_TYPE):
                properties = resource.Properties
                policy_document = getattr(properties, self.PROPERTY_WITH_POLICYDOCUMENT)
                for statement in policy_document._statement_as_list():
                    filters_available_context = {
                        "config": self._config,
                        "extras": extras,
                        "logical_id": logical_id,
                        "resource": resource,
                        "statement": statement,
                    }
                    self._do_statement_check(result, logical_id, statement, filters_available_context)
        return result

    def _do_statement_check(
        self, result: Result, logical_id: str, statement: Statement, filters_available_context: Dict
    ):
        if statement.Effect == "Allow":
            for principal in statement.get_principal_list():
                account_id = get_account_id_from_principal(principal)
                filters_available_context["principal"] = principal
                filters_available_context["account_id"] = account_id
                if (
                    # checks if principal is a canonical id and is allowed
                    principal not in self.valid_principals
                    # if it wasn't a canonical id and contains a valid account id
                    and account_id not in self.valid_principals
                    # if principal is an AWS service
                    and not principal.endswith(".amazonaws.com")
                ):
                    if statement.Condition and statement.Condition.dict():
                        logger.warning(
                            f"Not adding {type(self).__name__} failure in {logical_id} "
                            f"because there are conditions: {statement.Condition}"
                        )
                    elif not self._config.aws_account_id:
                        logger.warning(
                            f"Not adding {type(self).__name__} failure in {logical_id} "
                            f"because no AWS Account ID was found in the config."
                        )
                    elif principal.startswith("GETATT") or principal.startswith("UNDEFINED_"):
                        self.add_failure_to_result(
                            result,
                            self.REASON.format(logical_id, principal),
                            rule_mode=RuleMode.DEBUG,
                            resource_ids={logical_id},
                            context=filters_available_context,
                        )
                    else:
                        self.add_failure_to_result(
                            result,
                            self.REASON.format(logical_id, principal),
                            resource_ids={logical_id},
                            context=filters_available_context,
                        )


class CrossAccountTrustRule(CrossAccountCheckingRule):
    """
    Checks if the trust policy of a role grants permissions to principals from other accounts.
    Do not use whole accounts as principals.

    Risk:
        It might allow other AWS identities to escalate privileges.

    Fix:
        If cross account permissions are required, the stack should be added to the allowlist for this rule.
        Otherwise, the access should be removed from the CloudFormation definition.

    Filters context:
        | Parameter   | Type        | Description                                                    |
        |:-----------:|:-----------:|:--------------------------------------------------------------:|
        |`config`     | str         | `config` variable available inside the rule                    |
        |`extras`     | str         | `extras` variable available inside the rule                    |
        |`logical_id` | str         | ID used in Cloudformation to refer the resource being analysed |
        |`resource`   | `IAMRole`   | Resource that is being addressed                               |
        |`statement`  | `Statement` | Statement being checked found in the Resource                  |
        |`principal`  | `str`       | AWS Principal being checked found in the statement             |
        |`account_id` | `str`       | Account ID found in the principal                              |
    """

    REASON = "{} has forbidden cross-account trust relationship with {}"
    RESOURCE_TYPE = IAMRole
    PROPERTY_WITH_POLICYDOCUMENT = "AssumeRolePolicyDocument"


class S3CrossAccountTrustRule(CrossAccountCheckingRule):
    """
    Check for cross account access in S3 bucket policies. Cross account access by default should not be allowed.

    Risk:
        It might allow other AWS identities to access/modify content of the bucket.

    Fix:
        If cross account permissions are required for S3 access, the stack should be added to the allowlist for this rule.
        Otherwise, the access should be removed from the CloudFormation definition.

    Filters context:
        | Parameter   | Type             | Description                                                    |
        |:-----------:|:----------------:|:--------------------------------------------------------------:|
        |`config`     | str              | `config` variable available inside the rule                    |
        |`extras`     | str              | `extras` variable available inside the rule                    |
        |`logical_id` | str              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`   | `S3BucketPolicy` | Resource that is being addressed                               |
        |`statement`  | `Statement`      | Statement being checked found in the Resource                  |
        |`principal`  | `str`            | AWS Principal being checked found in the statement             |
        |`account_id` | `str`            | Account ID found in the principal                              |
    """

    REASON = "{} has forbidden cross-account policy allow with {} for an S3 bucket."
    RESOURCE_TYPE = S3BucketPolicy
    PROPERTY_WITH_POLICYDOCUMENT = "PolicyDocument"


class KMSKeyCrossAccountTrustRule(CrossAccountCheckingRule):
    """
    Checks for KMS keys that allow cross-account principals to get access to the key.

    Risk:
        It might allow other AWS identities to read/modify the secrets.

    Fix:
        If cross account permissions are required for KMS access, the stack should be added to the allowlist for this rule.
        Otherwise, the access should be removed from the CloudFormation definition.

    Filters context:
        | Parameter   | Type        | Description                                                    |
        |:-----------:|:-----------:|:--------------------------------------------------------------:|
        |`config`     | str         | `config` variable available inside the rule                    |
        |`extras`     | str         | `extras` variable available inside the rule                    |
        |`logical_id` | str         | ID used in Cloudformation to refer the resource being analysed |
        |`resource`   | `KMSKey`    | Resource that is being addressed                               |
        |`statement`  | `Statement` | Statement being checked found in the Resource                  |
        |`principal`  | `str`       | AWS Principal being checked found in the statement             |
        |`account_id` | `str`       | Account ID found in the principal                              |
    """

    REASON = "{} has forbidden cross-account policy allow with {} for an KMS Key Policy"
    RESOURCE_TYPE = KMSKey
    PROPERTY_WITH_POLICYDOCUMENT = "KeyPolicy"



import logging
import re
import sys
from io import TextIOWrapper
from pathlib import Path
from typing import Dict, List, Optional, Tuple

import click
import pycfmodel
from pycfmodel.model.cf_model import CFModel

from cfripper.__version__ import __version__
from cfripper.config.config import Config
from cfripper.config.pluggy.utils import get_all_rules
from cfripper.exceptions import FileEmptyException
from cfripper.model.enums import RuleMode
from cfripper.model.result import Result
from cfripper.model.utils import convert_json_or_yaml_to_dict
from cfripper.rule_processor import RuleProcessor

LOGGING_LEVELS = {
    "ERROR": logging.ERROR,
    "WARNING": logging.WARNING,
    "INFO": logging.INFO,
    "DEBUG": logging.DEBUG,
}


def setup_logging(level: str) -> None:
    logging.basicConfig(level=LOGGING_LEVELS[level], format="%(message)s")


def init_cfripper(
    rules_config_file: Optional[TextIOWrapper],
    rules_filters_folder: Optional[str],
    aws_account_id: Optional[str],
    aws_principals: Optional[List[str]],
) -> Tuple[Config, RuleProcessor]:
    rules = get_all_rules()
    config = Config(rules=rules.keys(), aws_account_id=aws_account_id, aws_principals=aws_principals)
    if rules_config_file:
        config.load_rules_config_file(rules_config_file)
    if rules_filters_folder:
        config.add_filters_from_dir(rules_filters_folder)
    rule_processor = RuleProcessor(*[rules.get(rule)(config) for rule in config.rules])
    return config, rule_processor


def get_cfmodel(template: TextIOWrapper) -> CFModel:
    template_file = convert_json_or_yaml_to_dict(template.read())
    if not template_file:
        raise FileEmptyException(f"{template.name} is empty and not a valid template.")
    cfmodel = pycfmodel.parse(template_file)
    return cfmodel


def analyse_template(cfmodel: CFModel, rule_processor: RuleProcessor, config: Config) -> Result:
    return rule_processor.process_cf_template(cfmodel, config)


def format_result_json(result: Result) -> str:
    return result.json()


def format_result_txt(result: Result) -> str:
    result_lines = [f"Valid: {result.valid}"]

    blocking_rules = result.get_failures(include_rule_modes={RuleMode.BLOCKING})
    if blocking_rules:
        result_lines.append("Issues found:")
        [result_lines.append(f"\t- {r.rule}: {r.reason}") for r in blocking_rules]

    monitoring_rules = result.get_failures(include_rule_modes={RuleMode.MONITOR})
    if monitoring_rules:
        result_lines.append("Monitored issues found:")
        [result_lines.append(f"\t- {r.rule}: {r.reason}") for r in monitoring_rules]

    return "\n".join(result_lines)


def format_result(result: Result, output_format: str) -> str:
    if output_format == "json":
        return format_result_json(result)
    else:
        return format_result_txt(result)


def save_to_file(path: Path, result: str) -> None:
    path.write_text(result)
    logging.info(f"Result saved in {path}")


def print_to_stdout(result: str) -> None:
    click.echo(result)


def output_handling(template_name: str, result: str, output_format: str, output_folder: Optional[str]) -> None:
    if output_folder:
        save_to_file(Path(output_folder) / f"{template_name}.cfripper.results.{output_format}", result)
    else:
        print_to_stdout(result)


def process_template(
    template: TextIOWrapper,
    resolve: bool,
    resolve_parameters: Optional[Dict],
    output_folder: Optional[str],
    output_format: str,
    rules_config_file: Optional[TextIOWrapper],
    rules_filters_folder: Optional[str],
    aws_account_id: Optional[str],
    aws_principals: Optional[List[str]],
) -> bool:
    logging.info(f"Analysing {template.name}...")

    cfmodel = get_cfmodel(template)
    if resolve:
        cfmodel = cfmodel.resolve(resolve_parameters)

    config, rule_processor = init_cfripper(rules_config_file, rules_filters_folder, aws_account_id, aws_principals)

    result = analyse_template(cfmodel, rule_processor, config)

    formatted_result = format_result(result, output_format)

    output_handling(template.name, formatted_result, output_format, output_folder)

    return result.valid


def validate_aws_account_id(ctx: click.Context, param: str, value: str) -> Optional[str]:
    if value in [None, ""]:
        return None
    if re.match("^[0-9]{12}$", str(value)):
        return str(value)
    else:
        raise click.BadParameter("AWS Account ID needs to be 12 digits – are you missing 0 prefixes?")


def validate_aws_principals(ctx: click.Context, param: str, value: str) -> Optional[List[str]]:
    if value in [None, ""]:
        return None
    return str(value).split(",")


@click.command()
@click.version_option(prog_name="cfripper", version=__version__)
@click.argument("templates", type=click.File("r"), nargs=-1)
@click.option(
    "--resolve/--no-resolve",
    is_flag=True,
    default=False,
    help="Resolves cloudformation variables and intrinsic functions",
    show_default=True,
)
@click.option(
    "--resolve-parameters",
    type=click.File("r"),
    help=(
        "JSON/YML file containing key-value pairs used for resolving CloudFormation files with templated parameters. "
        'For example, {"abc": "ABC"} will change all occurrences of {"Ref": "abc"} in the CloudFormation file to "ABC".'
    ),
)
@click.option(
    "--format",
    "output_format",
    type=click.Choice(["json", "txt"], case_sensitive=False),
    default="txt",
    help="Output format",
    show_default=True,
)
@click.option(
    "--output-folder",
    type=click.Path(exists=True, resolve_path=True, writable=True, file_okay=False),
    help="If not present, result will be sent to stdout",
)
@click.option(
    "--logging",
    "logging_level",
    type=click.Choice(LOGGING_LEVELS.keys(), case_sensitive=True),
    default="INFO",
    help="Logging level",
    show_default=True,
)
@click.option(
    "--rules-config-file", type=click.File("r"), help="Loads rules configuration file (type: [.py, .pyc])",
)
@click.option(
    "--rules-filters-folder",
    type=click.Path(exists=True, resolve_path=True, readable=True, file_okay=False),
    help="All files in the folder must be of type: [.py, .pyc]",
)
@click.option(
    "--aws-account-id",
    type=click.STRING,
    callback=validate_aws_account_id,
    help="A 12-digit AWS account number eg. 123456789012",
)
@click.option(
    "--aws-principals",
    type=click.STRING,
    callback=validate_aws_principals,
    help="A comma separated list of AWS principals eg. arn:aws:iam::123456789012:root,234567890123,"
    "arn:aws:iam::111222333444:user/user-name",
)
def cli(templates, logging_level, resolve_parameters, **kwargs):
    """
    Analyse AWS Cloudformation templates passed by parameter.
    Exit codes:
      - 0 = all templates valid and scanned successfully
      - 1 = error / issue in scanning at least one template
      - 2 = at least one template is not valid according to CFRipper (template scanned successfully)
      - 3 = unknown / unhandled exception in scanning the templates
    """
    try:
        setup_logging(logging_level)

        if kwargs["resolve"] and resolve_parameters:
            resolve_parameters = convert_json_or_yaml_to_dict(resolve_parameters.read())

        results_of_templates = [
            process_template(template=template, resolve_parameters=resolve_parameters, **kwargs)
            for template in templates
        ]
        sys.exit(2 if False in results_of_templates else 0)
    except FileEmptyException as file_empty:
        sys.exit(file_empty)
    except Exception as e:
        logging.exception(
            "Unhandled exception raised, please create an issue with the error message at "
            "https://github.com/Skyscanner/cfripper/issues"
        )
        try:
            sys.exit(e.errno)
        except AttributeError:
            sys.exit(3)


if __name__ == "__main__":
    cli()



__all__ = [
    "EC2SecurityGroupIngressOpenToWorldRule",
    "EC2SecurityGroupMissingEgressRule",
    "EC2SecurityGroupOpenToWorldRule",
]

from abc import ABC
from itertools import groupby
from operator import itemgetter
from typing import Dict, List, Optional, Tuple, Union

from pycfmodel.model.cf_model import CFModel
from pycfmodel.model.resources.properties.security_group_ingress_prop import SecurityGroupIngressProp
from pycfmodel.model.resources.security_group import SecurityGroup
from pycfmodel.model.resources.security_group_ingress import SecurityGroupIngress, SecurityGroupIngressProperties

from cfripper.model.enums import RuleGranularity
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule


class SecurityGroupOpenToWorldRule(Rule, ABC):
    """
    Base class not intended to be instantiated, but inherited from.
    This class provides common methods used to detect open ports.
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "Port(s) {} open to public IPs: ({}) in security group '{}'"

    def analyse_ingress(
        self,
        result: Result,
        logical_id: str,
        ingress: Union[SecurityGroupIngressProp, SecurityGroupIngressProperties],
        filters_available_context: Dict,
    ):
        if self.non_compliant_ip_range(ingress=ingress):
            open_ports = list(range(ingress.FromPort, ingress.ToPort + 1))
            non_allowed_open_ports = sorted(set(open_ports))

            if non_allowed_open_ports:
                ip_range = ingress.CidrIp or ingress.CidrIpv6
                filters_available_context["ingress_obj"] = ingress
                filters_available_context["ingress_ip"] = str(ip_range)
                filters_available_context["open_ports"] = open_ports
                filters_available_context["non_allowed_open_ports"] = non_allowed_open_ports

                self.add_failure_to_result(
                    result,
                    self.REASON.format(self.get_open_ports_wording(non_allowed_open_ports), ip_range, logical_id,),
                    resource_ids={logical_id},
                    context=filters_available_context,
                )

    def get_open_ports_wording(self, non_allowed_open_ports: List[int]) -> str:
        formatted_ports_range = []
        for port_range_start, port_range_end in self.get_open_ports_ranges(non_allowed_open_ports):
            if port_range_start == port_range_end:
                formatted_ports_range.append(f"{port_range_start}")
            else:
                formatted_ports_range.append(f"{port_range_start}-{port_range_end}")
        return ", ".join(formatted_ports_range)

    def get_open_ports_ranges(self, open_ports: List[int]) -> List[Tuple[int, int]]:
        open_ports_ranges = []
        for k, group in groupby(enumerate(open_ports), lambda x: x[1] - x[0]):
            port_range = list(map(itemgetter(1), group))
            open_ports_ranges.append((port_range[0], port_range[-1]))
        return open_ports_ranges

    def non_compliant_ip_range(
        self, ingress: Optional[Union[SecurityGroupIngressProp, SecurityGroupIngressProperties]]
    ) -> bool:
        if ingress is None:
            return False
        return (
            ingress.ipv4_slash_zero()
            or ingress.ipv6_slash_zero()
            or (ingress.CidrIp and ingress.CidrIp.is_global)
            or (ingress.CidrIpv6 and ingress.CidrIpv6.is_global)
        )


class EC2SecurityGroupOpenToWorldRule(SecurityGroupOpenToWorldRule):
    """
    Checks if security groups have an ingress IP that is open to the world for ports other than 80 and 443.
    All other ports should be closed off from public access to prevent a serious security misconfiguration.

    Fix:
        Most security groups only need to be [accessed privately](https://en.wikipedia.org/wiki/Private_network), and
        this can typically be done by specifying the CIDR of a Security Group's ingress to `10.0.0.0/8` or similar.

        Unless required, do not use the following IP ranges in your Security Group Ingress:

          - `0.0.0.0/0`.

          - Any `/8` that does not start with 10.

          - `172/8` or `192/8` (use `172.16/12` and `192.168/16` ranges, per RFC1918 specification).

        As per RFC4193, `fd00::/8` IPv6 addresses should be used to define a private network.

    Code for fix:
        ````json
        {
            "AWSTemplateFormatVersion": "2010-09-09",
            "Resources": {
                "SecurityGroup": {
                    "Type": "AWS::EC2::SecurityGroup",
                    "Properties": {
                        "GroupDescription": "description",
                        "SecurityGroupIngress": [
                            // this is compliant. Port 22 (typically SSH) is accessible from the private network only.
                            {
                                "IpProtocol": "tcp",
                                "CidrIp": "10.0.0.0/8",
                                "FromPort": 22,
                                "ToPort": 22
                            },
                            // this is not compliant. Anyone with the IP for this EC2 instance can connect on port 9090.
                            {
                                "IpProtocol": "tcp",
                                "CidrIp": "0.0.0.0/0",
                                "FromPort": 9090,
                                "ToPort": 9090
                            }
                        ]
                    }
                }
            }
        }
        ````

    Filters context:
        | Parameter               | Type                       | Description                                                    |
        |:-----------------------:|:--------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                        | `config` variable available inside the rule                    |
        |`extras`                 | str                        | `extras` variable available inside the rule                    |
        |`logical_id`             | str                        | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `SecurityGroup`            | Resource that is being addressed                               |
        |`ingress_ip`             | str                        | IP Address range (IpV4 or IpV6) of the ingress object          |
        |`ingress_obj`            | `SecurityGroupIngressProp` | SecurityGroupIngressProp being checked found in the Resource   |
        |`open_ports`             | `List[int]`                | List of all open ports defined                                 |
        |`non_allowed_open_ports` | `List[int]`                | List of all non allowed open ports defined                     |
    """

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.resources_filtered_by_type({SecurityGroup}).items():
            list_security_group_ingress = resource.Properties.SecurityGroupIngress
            if not isinstance(list_security_group_ingress, list):
                list_security_group_ingress = [list_security_group_ingress]
            for ingress in list_security_group_ingress:
                filters_available_context = {
                    "config": self._config,
                    "extras": extras,
                    "logical_id": logical_id,
                    "resource": resource,
                }
                self.analyse_ingress(result, logical_id, ingress, filters_available_context)
        return result


class EC2SecurityGroupIngressOpenToWorldRule(SecurityGroupOpenToWorldRule):
    """
    Checks if security groups have an ingress IP that is open to the world for ports other than 80 and 443.
    All other ports should be closed off from public access to prevent a serious security misconfiguration.

    Fix:
        Most security groups only need to be [accessed privately](https://en.wikipedia.org/wiki/Private_network), and
        this can typically be done by specifying the CIDR of a Security Group's ingress to `10.0.0.0/8` or similar.

        Unless required, do not use the following IP ranges in your Security Group Ingress:

          - `0.0.0.0/0`.

          - Any `/8` that does not start with 10.

          - `172/8` or `192/8` (use `172.16/12` and `192.168/16` ranges, per RFC1918 specification).

        As per RFC4193, `fd00::/8` IPv6 addresses should be used to define a private network.

    Code for fix:
        ````json
        {
            "AWSTemplateFormatVersion": "2010-09-09",
            "Resources": {
                // this is compliant. Port 22 (typically SSH) is accessible from the private network only.
                "InboundRule1": {
                    "Type": "AWS::EC2::SecurityGroupIngress",
                    "Properties": {
                        "GroupId": {
                            "Fn::GetAtt": [
                                "TargetSG",
                                "GroupId"
                            ]
                        },
                        "CidrIp": "10.0.0.0/8",
                        "IpProtocol": "tcp",
                        "FromPort": "22",
                        "ToPort": "22",
                        "SourceSecurityGroupId": "sg-12345678",
                        "SourceSecurityGroupOwnerId": "123456789012"
                    }
                },
                // this is not compliant. Anyone with the IP for this EC2 instance can connect on port 9090.
                "InboundRule2": {
                    "Type": "AWS::EC2::SecurityGroupIngress",
                    "Properties": {
                        "GroupId": {
                            "Fn::GetAtt": [
                                "TargetSG",
                                "GroupId"
                            ]
                        },
                        "CidrIp": "0.0.0.0/0",
                        "IpProtocol": "tcp",
                        "FromPort": "9090",
                        "ToPort": "9090",
                        "SourceSecurityGroupId": "sg-12345678",
                        "SourceSecurityGroupOwnerId": "123456789012"
                    }
                }
            }
        }
        ````

    Filters context:
        | Parameter               | Type                             | Description                                                    |
        |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
        |`config`                 | str                              | `config` variable available inside the rule                    |
        |`extras`                 | str                              | `extras` variable available inside the rule                    |
        |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
        |`resource`               | `SecurityGroupIngress`           | Resource that is being addressed                               |
        |`ingress_ip`             | str                              | IP Address range (IpV4 or IpV6) of the ingress object          |
        |`ingress_obj`            | `SecurityGroupIngressProperties` | SecurityGroupIngress being checked found in the Resource       |
        |`open_ports`             | `List[int]`                      | List of all open ports defined                                 |
        |`non_allowed_open_ports` | `List[int]`                      | List of all non allowed open ports defined                     |
    """

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.resources_filtered_by_type({SecurityGroupIngress}).items():
            filters_available_context = {
                "config": self._config,
                "extras": extras,
                "logical_id": logical_id,
                "resource": resource,
            }
            self.analyse_ingress(result, logical_id, resource.Properties, filters_available_context)
        return result


class EC2SecurityGroupMissingEgressRule(Rule):
    """
    Checks that Security Groups are defined with an egress policy, even if this is still allowing all
    outbound traffic.

    Risk:
        If no egress rule is specified, the default is to open all outbound traffic to the world. Whilst
        some services may need this, it is usually the case that the security group can be locked down
        more. A NAT instance for example may require a completely open egress policy.

        Allowing unrestricted (`0.0.0.0/0` or `::/0`) outbound/egress access can increase opportunities for
        malicious activity such as such as Denial of Service (DoS) attacks or Distributed Denial of Service (DDoS)
        attacks.

    Fix:
        Explicitly defining the egress policy for the security group.

    Code for fix:
        Even in the example below, the egress rule added will allow HTTP traffic out to the world. However,
        this will pass the rule.

        ````json
        // example from https://stelligent.com/2016/04/07/finding-security-problems-early-in-the-development-process-of-a-cloudformation-template-with-cfn-nag/
        {
            "Resources": {
                "sg": {
                    "Type": "AWS::EC2::SecurityGroup",
                    "Properties": {
                        "GroupDescription": "some_group_desc",
                        "SecurityGroupIngress": {
                            "CidrIp": "10.1.2.3/32",
                            "FromPort": 34,
                            "ToPort": 34,
                            "IpProtocol": "tcp"
                        },
                        // addition of egress to the `sg` resource
                        "SecurityGroupEgress": {
                            "CidrIp": "0.0.0.0/0",
                            "FromPort": 80,
                            "ToPort": 80,
                            "IpProtocol": "tcp"
                        },
                        "VpcId": "vpc-12345678"
                    }
                }
            }
        }
        ````

    Filters context:
        | Parameter   | Type            | Description                                                    |
        |:-----------:|:---------------:|:--------------------------------------------------------------:|
        |`config`     | str             | `config` variable available inside the rule                    |
        |`extras`     | str             | `extras` variable available inside the rule                    |
        |`logical_id` | str             | ID used in Cloudformation to refer the resource being analysed |
        |`resource`   | `SecurityGroup` | Resource that is being addressed                               |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = (
        "Missing egress rule in {} means all traffic is allowed outbound. Make this explicit if it is desired "
        "configuration"
    )

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if isinstance(resource, SecurityGroup) and not resource.Properties.SecurityGroupEgress:
                self.add_failure_to_result(
                    result,
                    self.REASON.format(logical_id),
                    resource_ids={logical_id},
                    context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
                )
        return result



from typing import Dict

from pluggy import HookspecMarker

from cfripper.rules.base_rules import Rule

hookspec = HookspecMarker("cfripper")


@hookspec
def cfripper_get_rules() -> Dict[str, Rule]:
    pass



__all__ = ["IAMRolesOverprivilegedRule", "IAMRoleWildcardActionOnPolicyRule"]

from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel
from pycfmodel.model.resources.iam_managed_policy import IAMManagedPolicy
from pycfmodel.model.resources.iam_role import IAMRole

from cfripper.config.regex import REGEX_IS_STAR, REGEX_WILDCARD_POLICY_ACTION
from cfripper.model.enums import RuleGranularity
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule


class IAMRolesOverprivilegedRule(Rule):
    """
    Rule that checks for wildcards in resources for a set of actions and restricts managed policies.

    Filters context:
    | Parameter               | Type                             | Description                                                    |
    |:-----------------------:|:--------------------------------:|:--------------------------------------------------------------:|
    |`config`                 | str                              | `config` variable available inside the rule                    |
    |`extras`                 | str                              | `extras` variable available inside the rule                    |
    |`logical_id`             | str                              | ID used in Cloudformation to refer the resource being analysed |
    |`resource`               | `IAMRole`                        | Resource that is being addressed                               |
    |`statement`              | `Optional[Statement]`            | Statement being checked found in the Resource                  |
    |`action`                 | `Optional[str]`                  | Action containing insecure permission with forbidden prefix    |
    """

    GRANULARITY = RuleGranularity.RESOURCE

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if isinstance(resource, IAMRole):
                self.check_managed_policies(result, logical_id, resource, extras)
                self.check_inline_policies(result, logical_id, resource, extras)
        return result

    def check_managed_policies(self, result: Result, logical_id: str, role: IAMRole, extras: Optional[Dict] = None):
        """Run the managed policies against a blacklist."""
        if not role.Properties.ManagedPolicyArns:
            return

        for managed_policy_arn in role.Properties.ManagedPolicyArns:
            if managed_policy_arn in self._config.forbidden_managed_policy_arns:
                self.add_failure_to_result(
                    result,
                    f"Role {logical_id} has forbidden Managed Policy {managed_policy_arn}",
                    resource_ids={logical_id},
                    context={
                        "config": self._config,
                        "extras": extras,
                        "logical_id": logical_id,
                        "resource": role,
                        "statement": None,
                        "action": None,
                    },
                )

    def check_inline_policies(self, result: Result, logical_id: str, role: IAMRole, extras: Optional[Dict] = None):
        """Check conditional and non-conditional inline policies."""
        if not role.Properties.Policies:
            return

        for policy in role.Properties.Policies:
            for statement in policy.PolicyDocument.statements_with(REGEX_IS_STAR):
                if statement.Effect and statement.Effect == "Allow":
                    for action in statement.get_action_list():
                        for prefix in self._config.forbidden_resource_star_action_prefixes:
                            if action.startswith(prefix):
                                self.add_failure_to_result(
                                    result,
                                    f"Role '{logical_id}' contains an insecure permission '{action}' in policy "
                                    f"'{policy.PolicyName}'",
                                    resource_ids={logical_id},
                                    context={
                                        "config": self._config,
                                        "extras": extras,
                                        "logical_id": logical_id,
                                        "resource": role,
                                        "statement": statement,
                                        "action": action,
                                    },
                                )


class IAMRoleWildcardActionOnPolicyRule(Rule):
    """
    Checks for use of wildcard characters in all IAM Role policies (including `AssumeRolePolicyDocument`)
    and [AWS Managed Policies](https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_managed-vs-inline.html).

    Filters context:
        | Parameter           | Type                               | Description                                                    |
        |:-------------------:|:----------------------------------:|:--------------------------------------------------------------:|
        |`config`             | str                                | `config` variable available inside the rule                    |
        |`extras`             | str                                | `extras` variable available inside the rule                    |
        |`logical_id`         | str                                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`           | `Union[IAMRole, IAMManagedPolicy]` | Resource that is being addressed                               |
    """

    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "IAM role {} should not allow a `*` action on its {}"

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if isinstance(resource, IAMRole):
                # check AssumeRolePolicyDocument.
                if resource.Properties.AssumeRolePolicyDocument.allowed_actions_with(REGEX_WILDCARD_POLICY_ACTION):
                    self.add_failure_to_result(
                        result,
                        self.REASON.format(logical_id, "AssumeRolePolicy"),
                        resource_ids={logical_id},
                        context={
                            "config": self._config,
                            "extras": extras,
                            "logical_id": logical_id,
                            "resource": resource,
                        },
                    )

                # check other policies of the IAM role.
                if resource.Properties.Policies:
                    for policy in resource.Properties.Policies:
                        if policy.PolicyDocument.allowed_actions_with(REGEX_WILDCARD_POLICY_ACTION):
                            self.add_failure_to_result(
                                result,
                                self.REASON.format(logical_id, f"{policy.PolicyName} policy"),
                                resource_ids={logical_id},
                                context={
                                    "config": self._config,
                                    "extras": extras,
                                    "logical_id": logical_id,
                                    "resource": resource,
                                },
                            )

            # check AWS::IAM::ManagedPolicy.
            elif isinstance(resource, IAMManagedPolicy) and resource.Properties.PolicyDocument.allowed_actions_with(
                REGEX_WILDCARD_POLICY_ACTION
            ):
                self.add_failure_to_result(
                    result,
                    self.REASON.format(logical_id, "AWS::IAM::ManagedPolicy"),
                    resource_ids={logical_id},
                    context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
                )
        return result



__all__ = ["S3BucketPolicyPrincipalRule"]
import logging
from typing import Dict, Optional

from pycfmodel.model.resources.s3_bucket_policy import S3BucketPolicy

from cfripper.model.enums import RuleGranularity, RuleRisk
from cfripper.model.result import Result
from cfripper.model.utils import get_account_id_from_principal
from cfripper.rules.base_rules import PrincipalCheckingRule, ResourceSpecificRule

logger = logging.getLogger(__file__)


class S3BucketPolicyPrincipalRule(PrincipalCheckingRule, ResourceSpecificRule):
    """
    Checks for non-allowed principals in S3 bucket policies.

    Risk:
        This is designed to block unintended access from third party accounts to your buckets.

    Fix:
        All principals connected to S3 Bucket Policies should be known. CFRipper checks that **all** principals meet
        the requirements expected. The list of valid accounts is defined in `valid_principals`, which is set in the config.

    Filters context:
        | Parameter   | Type               | Description                                                    |
        |:-----------:|:------------------:|:--------------------------------------------------------------:|
        |`config`     | str                | `config` variable available inside the rule                    |
        |`extras`     | str                | `extras` variable available inside the rule                    |
        |`logical_id` | str                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`   | `S3BucketPolicy`   | Resource that is being addressed                               |
        |`statement`  | `Statement`        | Statement being checked found in the Resource                  |
        |`principal`  | str                | AWS Principal being checked found in the statement             |
        |`account_id` | str                | Account ID found in the principal                              |
    """

    GRANULARITY = RuleGranularity.RESOURCE

    REASON = "S3 Bucket {} policy has non-allowed principals {}"
    RISK_VALUE = RuleRisk.HIGH
    RESOURCE_TYPES = (S3BucketPolicy,)

    def resource_invoke(self, resource: S3BucketPolicy, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()

        for statement in resource.Properties.PolicyDocument._statement_as_list():
            for principal in statement.get_principal_list():
                account_id = get_account_id_from_principal(principal)
                if not account_id:
                    continue
                if account_id not in self.valid_principals:
                    if statement.Condition and statement.Condition.dict():
                        logger.warning(
                            f"Not adding {type(self).__name__} failure in {logical_id} "
                            f"because there are conditions: {statement.Condition}"
                        )
                    else:
                        self.add_failure_to_result(
                            result,
                            self.REASON.format(logical_id, account_id),
                            resource_ids={logical_id},
                            context={
                                "config": self._config,
                                "extras": extras,
                                "logical_id": logical_id,
                                "resource": resource,
                                "statement": statement,
                                "principal": principal,
                                "account_id": account_id,
                            },
                        )
        return result



import logging
from datetime import datetime
from time import sleep
from typing import Dict, Optional

import boto3
from botocore.exceptions import ClientError

from .model.utils import convert_json_or_yaml_to_dict, extract_bucket_name_and_path_from_url

logger = logging.getLogger(__file__)


class Boto3Client:
    N_RETRIES = 5

    def __init__(self, account_id, region, stack_id):
        if not account_id or not region or not stack_id:
            raise Exception(f"Missing account_id or region: (Account: {account_id} - Region: {region})")

        arn = f"arn:aws:iam::{account_id}:role/cfripper-access"
        logger.info(f"Preparing to assume role: {arn}")

        client = boto3.client("sts")
        now = datetime.utcnow().isoformat().replace(":", ".")
        role_session_name = f"CfRipper{now}{stack_id.replace(':', '.')}"[:64]  # Limit of 64 chars
        response = client.assume_role(RoleArn=arn, RoleSessionName=role_session_name)
        self.session = boto3.Session(
            aws_access_key_id=response["Credentials"]["AccessKeyId"],
            aws_secret_access_key=response["Credentials"]["SecretAccessKey"],
            aws_session_token=response["Credentials"]["SessionToken"],
        )
        self.account_id = account_id
        self.region = region
        self.stack_id = stack_id

    def get_template(self) -> Optional[Dict]:
        client = self.session.client("cloudformation", region_name=self.region)
        stack_content = None
        i = 0
        while not stack_content and i < self.N_RETRIES:
            logger.info(f"Stack: {self.stack_id} on {self.account_id} - {self.region} get_template Attempt #{i}")
            try:
                response = client.get_template(StackName=self.stack_id)
                stack_content = response.get("TemplateBody")
                if not stack_content:
                    logger.warning(
                        f"No template body found for stack: {self.stack_id} on {self.account_id} - {self.region}"
                    )
                    sleep((i + 1) * 2)
            except ClientError as e:
                if e.response["Error"]["Code"] == "ValidationError":
                    logger.warning(f"There is no stack: {self.stack_id} on {self.account_id} - {self.region}")
                    return stack_content
                elif e.response["Error"]["Code"] == "Throttling":
                    logger.warning(f"AWS Throttling: {self.stack_id} on {self.account_id} - {self.region}")
                    sleep((i + 1) * 2)
                else:
                    logger.exception(
                        "Unexpected error occurred when getting stack template for:"
                        f" {self.stack_id} on {self.account_id} - {self.region}"
                    )
            i += 1
        # Fix when AWS doesn't return a dict
        # https://github.com/boto/botocore/issues/1058
        # https://github.com/boto/boto3/issues/1468
        return (
            convert_json_or_yaml_to_dict(stack_content, self.stack_id)
            if isinstance(stack_content, str)
            else stack_content
        )

    def download_template_to_dictionary(self, s3_url):
        """
        Download a CloudFormation template from S3 into a Dictionary.

        :param s3_url: The URL to download from.
        :return: Dictionary version of the CF Template.
        """
        bucket_name, file_path = extract_bucket_name_and_path_from_url(s3_url)

        client = self.session.client("s3", region_name=self.region)
        response = client.get_object(Bucket=bucket_name, Key=file_path)
        file_contents = response["Body"].read().decode("utf-8")

        return convert_json_or_yaml_to_dict(file_contents)

    def get_exports(self) -> Dict[str, str]:
        client = self.session.client("cloudformation", region_name=self.region)
        export_values = {}
        i = 0
        while not export_values and i < self.N_RETRIES:
            try:
                export_values = {export["Name"]: export["Value"] for export in client.list_exports().get("Exports", [])}
            except ClientError as e:
                if e.response["Error"]["Code"] == "AccessDenied":
                    logger.warning(
                        f"Access Denied for obtaining AWS Export values! ({self.account_id} - {self.region})"
                    )
                    return export_values
                elif e.response["Error"]["Code"] == "Throttling":
                    logger.warning(f"AWS Throttling: {self.stack_id} on {self.account_id} - {self.region}")
                    sleep((i + 1) * 2)
                else:
                    logger.exception(
                        f"Unhandled ClientError getting AWS Export values! ({self.account_id} - {self.region})"
                    )
            except Exception:
                logger.exception(f"Unknown exception getting AWS Export values! ({self.account_id} - {self.region})")
            i += 1
        return export_values



__all__ = ["KMSKeyWildcardPrincipalRule"]
import logging
import re
from typing import Dict, Optional

from pycfmodel.model.cf_model import CFModel
from pycfmodel.model.resources.kms_key import KMSKey

from cfripper.model.enums import RuleGranularity
from cfripper.model.result import Result
from cfripper.rules.base_rules import Rule

logger = logging.getLogger(__file__)


class KMSKeyWildcardPrincipalRule(Rule):
    """
    Check for wildcards in principals in KMS Policies.

    Filters context:
        | Parameter           | Type                               | Description                                                    |
        |:-------------------:|:----------------------------------:|:--------------------------------------------------------------:|
        |`config`             | str                                | `config` variable available inside the rule                    |
        |`extras`             | str                                | `extras` variable available inside the rule                    |
        |`logical_id`         | str                                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`           | `KMSKey`                           | Resource that is being addressed                               |
        |`statement`          | `Statement`                        | Statement being checked found in the Resource                  |
        |`principal`          | str                                | AWS Principal being checked found in the statement             |
    """

    GRANULARITY = RuleGranularity.RESOURCE

    REASON = "KMS Key policy {} should not allow wildcard principals"
    CONTAINS_WILDCARD_PATTERN = re.compile(r"^(\w*:)?\*$")

    def invoke(self, cfmodel: CFModel, extras: Optional[Dict] = None) -> Result:
        result = Result()
        for logical_id, resource in cfmodel.Resources.items():
            if isinstance(resource, KMSKey):
                for statement in resource.Properties.KeyPolicy._statement_as_list():
                    if statement.Effect == "Allow" and statement.principals_with(self.CONTAINS_WILDCARD_PATTERN):
                        for principal in statement.get_principal_list():
                            if self.CONTAINS_WILDCARD_PATTERN.match(principal):
                                if statement.Condition and statement.Condition.dict():
                                    logger.warning(
                                        f"Not adding {type(self).__name__} failure in {logical_id} "
                                        f"because there are conditions: {statement.Condition}"
                                    )
                                else:
                                    self.add_failure_to_result(
                                        result,
                                        self.REASON.format(logical_id),
                                        resource_ids={logical_id},
                                        context={
                                            "config": self._config,
                                            "extras": extras,
                                            "logical_id": logical_id,
                                            "resource": resource,
                                            "statement": statement,
                                            "principal": principal,
                                        },
                                    )
        return result



from pathlib import Path

from setuptools import find_packages, setup

from cfripper.__version__ import __version__

project_root_path = Path(__file__).parent

install_requires = [
    "boto3>=1.4.7,<2",
    "cfn_flip>=1.2.0",
    "click~=7.1.1",
    "pluggy~=0.13.1",
    "pycfmodel>=0.11.0",
    "pydash~=4.7.6",
    "PyYAML>=4.2b1",
]

dev_requires = [
    "black==19.10b0",
    "flake8>=3.3.0",
    "isort==4.3.21",
    "pytest>=3.6",
    "pytest-cov>=2.5.1",
    "pip-tools>=5.3.1",
    "moto==1.3.13",
]

docs_requires = [
    "ansi2html==1.5.2",
    "Markdown==3.2.2",
    "markdown-include==0.5.1",
    "mkdocs-exclude==1.0.2",
    "mkdocs-macros-plugin==0.4.9",
    "mkdocs-material==5.5.12",
    "mkdocs-material-extensions==1.0",
    "mkdocs-minify-plugin==0.3.0",
    "mkdocs==1.1.2",
    "pygments==2.5.2",
]

setup(
    name="cfripper",
    version=__version__,
    author="Skyscanner Product Security",
    author_email="security@skyscanner.net",
    entry_points={"console_scripts": ["cfripper=cfripper.cli:cli"]},
    long_description=(project_root_path / "README.md").read_text(),
    long_description_content_type="text/markdown",
    url="https://github.com/Skyscanner/cfripper",
    description="Library and CLI tool for analysing CloudFormation templates and check them for security compliance.",
    packages=find_packages(exclude=("docs", "tests")),
    platforms="any",
    python_requires=">=3.7",
    install_requires=install_requires,
    tests_require=dev_requires,
    extras_require={"dev": dev_requires, "docs": docs_requires},
)



__all__ = ["S3ObjectVersioningRule"]

from typing import Dict, Optional

from pycfmodel.model.resources.s3_bucket import S3Bucket

from cfripper.model.enums import RuleGranularity, RuleRisk
from cfripper.model.result import Result
from cfripper.rules.base_rules import ResourceSpecificRule


class S3ObjectVersioningRule(ResourceSpecificRule):
    """
    Checks if the S3 bucket has object versioning enabled or not.

    Risk:
        Not having this property enabled could make the bucket more vulnerable to ransomware attacks.
        Bucket versioning allows the automatic creation of multiple versions of an object.
        When an object is deleted with versioning turned on, it is only marked as deleted but is still retrievable.

    Fix:
        Add `VersioningConfiguration` property with the value `Enabled` to bucket as defined in the
        [AWS documentation](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-versioningconfig.html).

    Code for fix:
        ````yml
        Resources:
          S3Bucket:
            Type: AWS::S3::Bucket
            Properties:
              ...
              VersioningConfiguration:
                Status: Enabled
              ...
        ````

    Filters context:
        | Parameter     | Type               | Description                                                    |
        |:-------------:|:------------------:|:--------------------------------------------------------------:|
        |`config`       | str                | `config` variable available inside the rule                    |
        |`extras`       | str                | `extras` variable available inside the rule                    |
        |`logical_id`   | str                | ID used in Cloudformation to refer the resource being analysed |
        |`resource`     | `S3Bucket`         | Resource that is being addressed                               |
    """

    ENABLED_STATUS = "Enabled"
    GRANULARITY = RuleGranularity.RESOURCE
    REASON = "S3 Bucket {} is required to have object versioning enabled"
    RESOURCE_TYPES = (S3Bucket,)
    RISK_VALUE = RuleRisk.LOW

    def resource_invoke(self, resource: S3Bucket, logical_id: str, extras: Optional[Dict] = None) -> Result:
        result = Result()
        version_configuration = resource.Properties.VersioningConfiguration
        if version_configuration is None or version_configuration.get("Status") != self.ENABLED_STATUS:
            self.add_failure_to_result(
                result,
                self.REASON.format(logical_id),
                resource_ids={logical_id},
                context={"config": self._config, "extras": extras, "logical_id": logical_id, "resource": resource},
            )
        return result
